@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: fwbedit.cpp,v 1.11 2004/04/11 07:18:18 vkurland Exp $
+  $Id: fwbedit.cpp,v 1.12 2004/11/26 03:59:26 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -45,7 +45,16 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <ctype.h>
-#include <getopt.h>
+
+#ifdef HAVE_GETOPT_H
+#  include <getopt.h>
+#else
+#  ifdef _WIN32
+#    include <getopt.h>
+#  else
+#    include <stdlib.h>
+#  endif
+#endif
 
 #include "fwbuilder/Resources.h"
 
