@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWBSettings.h,v 1.17 2004/06/11 15:11:32 vkurland Exp $
+  $Id: FWBSettings.h,v 1.18 2005/01/22 01:58:00 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,6 +28,7 @@
 
 #include <qsettings.h>
 #include <qrect.h>
+#include <qprinter.h>
 
 class QWidget;
 
@@ -132,7 +133,9 @@ class FWBSettings : public QSettings {
     
     QString getSSHPath();
     void    setSSHPath(const QString &path);
-    
+
+    void    getPrinterOptions(QPrinter *printer,int &pageWidth,int &pageHeight);
+    void    setPrinterOptions(QPrinter *printer,int pageWidth,int pageHeight);
 };
 
 #endif
