@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindow.cpp,v 1.134 2004/09/28 05:21:00 vkurland Exp $
+  $Id: FWWindow.cpp,v 1.154 2005/04/24 06:51:01 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -104,6 +104,8 @@
 #  include <unistd.h>     // for access(2)
 #endif
 
+#include <algorithm>
+
 #include <qaction.h>
 #include <qlistview.h>
 #include <qmessagebox.h>
@@ -129,6 +131,7 @@
 #include <qtimer.h>
 #include <qstatusbar.h>
 #include <qlabel.h>
+#include <qprinter.h>
 
 using namespace libfwbuilder;
 using namespace std;
@@ -142,6 +145,10 @@ FWWindow::FWWindow() : FWBMainWindow_q()
     visibleFirewall = NULL;
     delete treeFrame;
 
+    printer = new QPrinter(QPrinter::HighResolution);
+
+    noFirewalls = tr("No firewalls defined");
+
     autosaveTimer = new QTimer(this);
 
 // om is a global var
@@ -227,10 +234,17 @@ void FWWindow::info(FWObject *obj)
 
         QString s="";
         if (st->getInfoStyle()==2)
+        {
             s=FWObjectPropertiesFactory::getObjectPropertiesDetailed(obj)
                 + QString("<hr height=\"0\">");
-
-        s+=QString::fromUtf8(obj->getComment().c_str());
+            oi->append(s);
+        }
+        oi->setTextFormat(QTextEdit::PlainText);
+        oi->setBold(false);
+        oi->setItalic(false);
+        oi->setUnderline(false);
+        oi->setColor(Qt::black);
+        s=QString::fromUtf8(obj->getComment().c_str());
         oi->append(s);
         oi->setCursorPosition(0,0);
     }
@@ -304,7 +318,7 @@ QString FWWindow::chooseNewFileName(const QString &fname,
                  this,"Firewall Builder", 
                  tr("The file %1 already exists.\nDo you want to overwrite it ?")
                  .arg(fn.latin1()),
-                 "&Yes", "&No", QString::null,
+                 tr("&Yes"), tr("&No"), QString::null,
                  0, 1 )==0 )
     {
         return fn;
@@ -336,20 +350,18 @@ void FWWindow::fileProp()
 
 void FWWindow::fileNew()
 {    
-    if (!saveIfModified()) return;
-
-    if (!systemFile && rcs!=NULL) fileClose();  // fileClose calls load(this)
-    else  load(this);
-
-    visibleFirewall = NULL;
-
-    showFirewalls();
-
     QString nfn=chooseNewFileName(st->getWDir(),true,
                                   tr("Choose name and location for the new file"));
-
     if ( !nfn.isEmpty() )
     {
+        if (!saveIfModified() || !checkin(true)) return;
+        if (!systemFile && rcs!=NULL) fileClose();  // fileClose calls load(this)
+        else  load(this);
+
+        visibleFirewall = NULL;
+        showFirewalls();
+
+
         setFileName(nfn);
 
         save();
@@ -364,14 +376,13 @@ void FWWindow::fileNew()
 
 void FWWindow::fileOpen()
 {    
-    if (!saveIfModified()) return;
-
-    if (!systemFile && rcs!=NULL) fileClose();
-
     RCSFileDialog   fd(this, 0, true);
 
     if ( fd.exec()== QDialog::Accepted )
     {
+        if (!saveIfModified() || !checkin(true)) return;
+        if (!systemFile && rcs!=NULL) fileClose();
+
         RCS *rcs = fd.getSelectedRev();
 
         if (rcs==NULL) return;
@@ -407,28 +418,16 @@ void FWWindow::fileClose()
 
     if (oe->isVisible()) oe->hide();
 
-    if (!saveIfModified()) return;
-
-    if (fwbdebug) qDebug("FWWindow::fileClose(): will save");
-
-/***********************************************************************
- * TODO : add an option "RCS support"
- */
-    if (!systemFile && rcs && rcs->isCheckedOut() && !rcs->isTemp())
-    {
-        if (fwbdebug) qDebug("FWWindow::fileClose(): checking in");
-        if (!checkin(true)) return;
-        if (fwbdebug) qDebug("FWWindow::fileClose(): done");
-        delete rcs;
-    }
-/***********************************************************************/
+    if (!saveIfModified() || !checkin(true))  return;
 
+    if (rcs) delete rcs;
     rcs=NULL;
 
     if (fwbdebug) qDebug("FWWindow::fileClose(): clearing widgets");
 
     firewalls.clear();
     fwList->clear();
+    visibleFirewall = NULL;
     clearFirewallTabs();
     ruleSetViews.clear();
     om->clearObjects();
@@ -464,15 +463,9 @@ void FWWindow::fileSaveAs()
  */
     if (!systemFile && rcs &&
         !rcs->isRO() && !rcs->isTemp() && !rcs->getFileName().isEmpty() &&
-        !saveIfModified()) return;
+        (!saveIfModified() || !checkin(true))
+    ) return;
 
-    if (!systemFile && rcs && rcs->isCheckedOut() && !rcs->isTemp())
-    {
-        if (fwbdebug) qDebug("FWWindow::fileSaveAs(): checking in");
-        if (!checkin(true)) return;
-        if (fwbdebug) qDebug("FWWindow::fileSaveAs(): done");
-    }
-    
     QString oldFileName = rcs->getFileName();
     if (rcs!=NULL) delete rcs;
 
@@ -493,19 +486,11 @@ void FWWindow::fileSaveAs()
     }
 }
 
-void FWWindow::filePrint()
-{
-}
-
 void FWWindow::fileExit()
 {    
-    if (saveIfModified())
+    if (saveIfModified() && checkin(true))
     {
-        if (!systemFile && rcs && rcs->isCheckedOut())
-        {
-            if (!checkin(true)) return;
-            delete rcs;
-        }
+        if (rcs) delete rcs;
         qApp->quit();
     }
 }
@@ -528,19 +513,35 @@ void FWWindow::fileDiscard()
     {
 /* need to close the file without asking and saving, then reopen it again */
         
-//        QString fname = rcs->getFileName();
+        QString fname = rcs->getFileName();
 
         db()->setDirty(false);  // so it wont ask if user wants to save
         rcs->abandon();
         fileClose();
 
-        load(this);
+        try
+        {
+            RCS *rcs = new RCS(fname);
+            if (rcs==NULL) return;
+            rcs->co();
+            load(this, rcs );
+        } catch (FWException &ex)
+        {
+/* if there was an exception, abort operation. E.g. RCS::co may throw
+ * exception */
+            load(this);
+            return;
+        }
+/***********************************************************************/
+
+        showFirewalls();
     }
 }
 
 void FWWindow::fileAddToRCS()
 {
     if (!saveIfModified()) return;
+    if (rcs && rcs->isCheckedOut()) return;
 
     try
     {
@@ -551,7 +552,7 @@ void FWWindow::fileAddToRCS()
             QMessageBox::information(
                 this,"Firewall Builder", 
                 tr("File %1 has been added to RCS.").arg(rcs->getFileName()),
-                "&Continue", QString::null,QString::null,
+                tr("&Continue"), QString::null,QString::null,
                 0, 1 );
         }
     }
@@ -560,7 +561,7 @@ void FWWindow::fileAddToRCS()
         QMessageBox::critical(
             this,"Firewall Builder", 
             tr("Error adding file to RCS:\n%1").arg(ex.toString().c_str()),
-            "&Continue", QString::null,QString::null,
+            tr("&Continue"), QString::null,QString::null,
             0, 1 );
     }
 
@@ -624,12 +625,13 @@ void FWWindow::load(QWidget *dialogs_parent)
 
         for (list<libData>::iterator i=addOnLibs->begin(); i!=addOnLibs->end(); ++i)
         {
-            string libfname = i->path;
+            string libfname = i->path.latin1();
             if (libfname!=sysfname && i->load)
             {
                 FWObjectDatabase *ndb = new FWObjectDatabase();
                 ndb->load(libfname, &upgrade_predicate,librespath);
-                FWObject *dobj = ndb->getById( FWObjectDatabase::getDeletedObjectsId());
+                FWObject *dobj =
+                    ndb->getById( FWObjectDatabase::getDeletedObjectsId());
                 if (dobj) ndb->remove(dobj, false);
 
                 MergeConflictRes mcr(dlgp);
@@ -665,7 +667,7 @@ void FWWindow::load(QWidget *dialogs_parent)
         QMessageBox::warning(
             this,"Firewall Builder", 
             tr("Error loading file:\n%1").arg(ex.toString().c_str()),
-            "&Continue", QString::null,QString::null,
+            tr("&Continue"), QString::null,QString::null,
             0, 1 );
     }
 
@@ -722,6 +724,16 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
 /* loadingLib is true if user wants to open a library or master library file */
         bool loadingLib         = editingLibrary();
 
+        if (fwbdebug)
+        {
+            list<FWObject*> ll = ndb->getByType(Library::TYPENAME);
+            for (FWObject::iterator i=ll.begin(); i!=ll.end(); i++)
+            {
+                qDebug("* Found library %s %s in the data file",
+                       (*i)->getId().c_str(),(*i)->getName().c_str() );
+            }
+        }
+
 /* if user opens library file, clear read-only flag so they can edit it */
         if (loadingLib)
         {
@@ -738,9 +750,12 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
             for (list<libData>::iterator i=addOnLibs->begin();
                  i!=addOnLibs->end(); ++i)
             {
-                string libfname = i->path;
+                string libfname = i->path.latin1();
                 if (libfname!=sysfname && i->load)
                 {
+                    if (fwbdebug)
+                        qDebug("* Adding library %s",i->name.latin1());
+
                     FWObjectDatabase *ndb1 = new FWObjectDatabase();
                     ndb1->load(libfname, &upgrade_predicate,librespath);
                     FWObject  *nlib1 = ndb1->getFirstByType(Library::TYPENAME);
@@ -751,7 +766,8 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
                         assert(nlib1!=NULL);
                     }
                     string nlib1ID = nlib1->getId();
-                    FWObject *dobj = ndb1->getById( FWObjectDatabase::getDeletedObjectsId());
+                    FWObject *dobj =
+                        ndb1->getById(FWObjectDatabase::getDeletedObjectsId());
                     if (dobj) ndb1->remove(dobj, false);
 
                     MergeConflictRes mcr(dlgp);
@@ -780,6 +796,18 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
  */
         objdb = FWObjectDatabase::db;
 
+        if (fwbdebug)
+        {
+            qDebug("* Merge is done");
+            list<FWObject*> ll = db()->getByType(Library::TYPENAME);
+            for (FWObject::iterator i=ll.begin(); i!=ll.end(); i++)
+            {
+                qDebug("* Library %s %s in the data file",
+                       (*i)->getId().c_str(),(*i)->getName().c_str() );
+            }
+        }
+
+
 /* this is a hack: 'Standard' library should be read-only. I have too
  * many files I already converted to the new API/DTD and I am too lazy
  * to convert them again, so I patch it up here.
@@ -801,7 +829,13 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
 
         if ( ofinfo.extension(false)=="xml")
         {
-            QString nfn=ofinfo.baseName(true) + ".fwb";
+            if (fwbdebug)
+            {
+                qDebug("Need to rename file:  %s",fn.ascii());
+                qDebug("             dirPath: %s",ofinfo.dirPath(true).ascii());
+                qDebug("            filePath: %s",ofinfo.absFilePath().ascii());
+            }
+            QString nfn=ofinfo.dirPath(true) + "/" + ofinfo.baseName(true) + ".fwb";
 
             bool needToRename = true;
 
@@ -818,7 +852,6 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
 
             if (needToRename)
             {
-                bool firstDlg=true;
                 if (nfinfo.exists())
                 {
 /* .fwb file exists but .xml is not a symlink
@@ -834,31 +867,30 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
  * shorted version of the dialog and will be presented with "save
  * file" dialog again.
  */
-                    do
-                    {
-                        if (firstDlg)
-                        {
-                            QMessageBox::warning(
-                                this,"Firewall Builder", 
-                                tr("Firewall Builder 2 uses file extension '.fwb' and \nneeds to rename old data file '%1' to '%2',\nbut file '%3' already exists.\nChoose a different name for the new file.")
-                                .arg(fn).arg(nfn).arg(nfn),
-                                "&Continue", QString::null,QString::null,
-                                0, 1 );
-                        }
-                        firstDlg=false;
+                    QMessageBox::warning(
+                        this,"Firewall Builder", 
+                        tr("Firewall Builder 2 uses file extension '.fwb' and \nneeds to rename old data file '%1' to '%2',\nbut file '%3' already exists.\nChoose a different name for the new file.")
+                        .arg(fn).arg(nfn).arg(nfn),
+                        tr("&Continue"), QString::null,QString::null,
+                        0, 1 );
 
-                        nfn=chooseNewFileName(fn,true,
-                                              tr("Choose name and location for the new file"));
-                        if (!nfn.isEmpty())    nfinfo.setFile(nfn);
-                        else 
-                        {
-                            QMessageBox::warning(
-                                this,"Firewall Builder", 
-                                tr("Please choose a different name for the new file."),
-                                "&Continue", QString::null,QString::null,
-                                0, 1 );
-                        }
-                    } while (nfn.isEmpty());
+                    nfn=chooseNewFileName(fn,true,
+                                          tr("Choose name and location for the new file"));
+                    if (nfn.isEmpty())
+                    {
+                        QString oldFileName = ofinfo.absFilePath() + ".bak";
+                        rename(oldFileName.latin1(), fn.latin1());
+
+                        QMessageBox::warning(
+                            this,"Firewall Builder", 
+                            tr("Load operation cancelled and data file reverted to original version."),
+                            tr("&Continue"), QString::null,QString::null,
+                            0, 1 );
+
+                        load(this);
+                        return;
+                    }
+                    nfinfo.setFile(nfn);
                 }
 
                 rename(fn.latin1(), nfn.latin1());
@@ -868,7 +900,7 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
                 this,"Firewall Builder", 
                 tr("Firewall Builder 2 uses file extension '.fwb'. Your data file '%1' \nhas been renamed '%2'")
                 .arg(fn).arg(nfn),
-                "&Continue", QString::null,QString::null,
+                tr("&Continue"), QString::null,QString::null,
                 0, 1 );
                 
             }
@@ -905,13 +937,13 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
              QMessageBox::warning(
                  this,"Firewall Builder", 
                  tr("Error loading file:\n%1").arg(msg),
-                 "&Continue", QString::null,QString::null,
+                 tr("&Continue"), QString::null,QString::null,
                  0, 1 );
         } else
              QMessageBox::warning(
                  this,"Firewall Builder", 
                  tr("Error loading file:\n%1").arg(ex.toString().c_str()),
-                 "&Continue", QString::null,QString::null,
+                 tr("&Continue"), QString::null,QString::null,
                  0, 1 );
 
         load(this);
@@ -933,45 +965,46 @@ bool FWWindow::checkin(bool unlock)
 /* doing checkin only if we did checkout so rcs!=NULL */
     QString rlog="";
 
-    if (rcs && rcs->isCheckedOut() && !rcs->isTemp())
-    { 
-        if (rcs->isDiff()) // if the file hasn't changed, do not need to ask for the comment
-        {
-            if ( ! st->getRCSLogState())
-            {
-                RCSFileSaveDialog_q fsd(this);
-                if ( fsd.exec()== QDialog::Rejected ) return false;
+    if (systemFile || rcs==NULL || !rcs->isCheckedOut() || rcs->isTemp())
+        return true;
 
-                bool    empty_rcslog = fsd.nolog->isChecked();
-                if (empty_rcslog)
-                {
-                    rlog = "";
-                    st->setRCSLogState(true);
-                } else              rlog = fsd.rcslog->text();
-            }
-        }
-
-/***********************************************************************/
-        try
-        {
-            if (fwbdebug)  qDebug("about to check the file in");
-            rcs->ci(rlog,unlock);
-            if (fwbdebug)  qDebug("done");
-        }
-        catch (FWException &ex)
-        {
-            QMessageBox::warning(
-                this,"Firewall Builder", 
-                tr("Error checking in file %1:\n%2")
-                .arg(rcs->getFileName()).arg(ex.toString().c_str()),
-                "&Continue", QString::null, QString::null,
-                0, 1 );
-        }
-/***********************************************************************/
-    } else
+    if (rcs->isDiff()) // if the file hasn't changed, do not need to ask for the comment
     {
-        return false;
+        if ( ! st->getRCSLogState())
+        {
+            RCSFileSaveDialog_q fsd(this);
+            fsd.checkinDialogTitle->setText(
+                QString("<b>")+tr("Checking file %1 in RCS").arg(rcs->getFileName())+QString("</b>")
+            );
+            if ( fsd.exec()== QDialog::Rejected ) return false;
+
+            bool    empty_rcslog = fsd.nolog->isChecked();
+            if (empty_rcslog)
+            {
+                rlog = "";
+                st->setRCSLogState(true);
+            } else
+                rlog = fsd.rcslog->text();
+        }
     }
+
+/***********************************************************************/
+    try
+    {
+        if (fwbdebug)  qDebug("about to check the file in");
+        rcs->ci(rlog,unlock);
+        if (fwbdebug)  qDebug("done");
+    }
+    catch (FWException &ex)
+    {
+        QMessageBox::warning(
+            this,"Firewall Builder", 
+            tr("Error checking in file %1:\n%2")
+            .arg(rcs->getFileName()).arg(ex.toString().c_str()),
+            tr("&Continue"), QString::null, QString::null,
+            0, 1 );
+    }
+/***********************************************************************/
     return true;
 }
 
@@ -1052,7 +1085,7 @@ void FWWindow::save()
                 this,"Firewall Builder", 
                 tr("Error saving file %1: %2")
                 .arg(rcs->getFileName()).arg(err),
-                "&Continue", QString::null, QString::null,
+                tr("&Continue"), QString::null, QString::null,
                 0, 1 );
         }
     }
@@ -1067,6 +1100,44 @@ void FWWindow::loadLibrary(const string &libfpath)
         FWObjectDatabase *ndb = new FWObjectDatabase();
         ndb->load(libfpath,  &upgrade_predicate,  librespath);
 
+        FWObject *dobj = ndb->getById(FWObjectDatabase::getDeletedObjectsId());
+        if (dobj) ndb->remove(dobj, false);
+
+        list<FWObject*> newLibs;
+        newLibs= ndb->getByType(Library::TYPENAME);
+
+        list<FWObject*> currentLibs;
+        currentLibs= db()->getByType(Library::TYPENAME);
+
+        list<FWObject*> duplicateLibs;
+
+        for (list<FWObject*>::iterator i=newLibs.begin(); i!=newLibs.end(); i++)
+        {
+            string newLibID = (*i)->getId();
+            if (newLibID==STANDARD_LIB)
+            {
+                duplicateLibs.push_back(*i);
+                continue;
+            }
+            QString name = (*i)->getName().c_str();
+            if (std::find_if(currentLibs.begin(),currentLibs.end(),
+                             findFWObjectIDPredicate(newLibID))!=currentLibs.end() )
+            {
+                QMessageBox::warning(
+                    NULL,"Firewall Builder", 
+                    QObject::tr("Duplicate library '%1'").arg(QString::fromUtf8(name)),
+                    QObject::tr("&Continue"), QString::null,QString::null,
+                    0, 1 );
+                duplicateLibs.push_back(*i);
+            }
+        }
+    
+        if (!duplicateLibs.empty())
+        {
+            for (list<FWObject*>::iterator i=duplicateLibs.begin(); i!=duplicateLibs.end(); i++)
+                ndb->remove(*i,false);
+        }
+
         MergeConflictRes mcr(this);
         db()->merge(ndb, &mcr);
 
@@ -1078,7 +1149,7 @@ void FWWindow::loadLibrary(const string &libfpath)
             this,"Firewall Builder", 
             tr("Error loading file %1:\n%2").
                  arg(libfpath.c_str()).arg(ex.toString().c_str()),
-            "&Continue", QString::null,QString::null,
+            tr("&Continue"), QString::null,QString::null,
             0, 1 );
     }
 }
@@ -1099,7 +1170,7 @@ void FWWindow::fileImport()
     om->loadObjects();
     showFirewalls();
 
-    addOnLibs->add( fname.latin1() );
+//    addOnLibs->add( fname.latin1() );
 }
 
 void FWWindow::fileExport()
@@ -1138,7 +1209,7 @@ void FWWindow::showFirewalls()
 
     if (fl.size()==0)
     {
-        fwList->insertItem( tr("No firewalls defined") );
+        fwList->insertItem( noFirewalls );
         return;
     }
 
@@ -1171,8 +1242,14 @@ void FWWindow::addFirewallToList(FWObject *o)
     if (fwbdebug) qDebug("FWWindow::addFirewallToList %d %p %s",
                          n, o, o->getName().c_str() );
 
+    if (fwList->currentText() == noFirewalls )
+    {
+        fwList->removeItem(0);
+    }
+
     fwList->insertItem( QPixmap::fromMimeSource( icn_filename ),
-			QString::fromUtf8(o->getName().c_str()) );
+                        QString::fromUtf8(o->getName().c_str()) );
+
     firewalls.push_back(o);
 
     fwList->setCurrentItem( n );
@@ -1199,15 +1276,18 @@ void FWWindow::removeFirewallFromList(FWObject *o)
 
 void FWWindow::showFirewall(FWObject *obj)
 {
-    vector<FWObject*>::iterator i;
-    int n=0;
-    for (i=firewalls.begin(); i!=firewalls.end(); i++,n++)
+    if (firewalls.size()>0)
     {
-        if ( (*i)->getId()==obj->getId() )
+        vector<FWObject*>::iterator i;
+        int n=0;
+        for (i=firewalls.begin(); i!=firewalls.end(); i++,n++)
         {
-            fwList->setCurrentItem( n );
-            openFirewall( n );
-            return;
+            if ( (*i)->getId()==obj->getId() )
+            {
+                fwList->setCurrentItem( n );
+                openFirewall( n );
+                return;
+            }
         }
     }
 }
@@ -1227,6 +1307,13 @@ void FWWindow::ensureObjectVisible(FWReference *obj)
 
     RuleSetView *rsv = ruleSetViews[p];
 
+    if (rsv==NULL)
+    {
+        if (fwbdebug)
+            qDebug("FWWindow::ensureObjectVisible : orphan rule set found");
+        return;
+    }
+
     ruleSets->showPage(rsv);
 
     rsv->selectRE( obj );
@@ -1303,7 +1390,7 @@ void FWWindow::reopenFirewall()
     RuleSetView *rsv;
 
     Policy *pol=Policy::cast(visibleFirewall->getFirstByType(Policy::TYPENAME));
-    ruleSets->addTab( rsv=new PolicyView(pol) , tr("Policy") );
+    ruleSets->addTab( rsv=new PolicyView(pol,NULL) , tr("Policy") );
     ruleSetViews[pol]=rsv;
 
     FWObjectTypedChildIterator j=visibleFirewall->findByType(Interface::TYPENAME);
@@ -1317,12 +1404,12 @@ void FWWindow::reopenFirewall()
             tabName=QString::fromUtf8(intf->getLabel().c_str());
         else
             tabName=QString::fromUtf8(intf->getName().c_str());
-        ruleSets->addTab( rsv=new InterfacePolicyView(ip) , tabName );
+        ruleSets->addTab( rsv=new InterfacePolicyView(ip,NULL) , tabName );
         ruleSetViews[ip]=rsv;
     }
 
     NAT *nat  = NAT::cast(visibleFirewall->getFirstByType(NAT::TYPENAME));
-    ruleSets->addTab( rsv=new NATView(nat) , tr("NAT") );
+    ruleSets->addTab( rsv=new NATView(nat,NULL) , tr("NAT") );
     ruleSetViews[nat]=rsv;
 
     ruleSets->setCurrentPage( currentPage );
@@ -1346,11 +1433,15 @@ void FWWindow::reopenFirewall()
 
 void FWWindow::openFirewall( int idx )
 {
-    FWObject *fw=firewalls[idx];
-    visibleFirewall = fw;
-    firewallName->setText( fw->getName().c_str() );
-    reopenFirewall();
-    om->openObject(fw);
+    if (firewalls.size()>0)
+    {
+        FWObject *fw=firewalls[idx];
+        visibleFirewall = fw;
+        firewallName->setText( fw->getName().c_str() );
+        reopenFirewall();
+        om->openObject(fw);
+    } else
+        visibleFirewall = NULL;
 }
 
 void FWWindow::selectRules()
@@ -1443,17 +1534,9 @@ void FWWindow::editPrefs()
 
 void FWWindow::closeEvent( QCloseEvent * ev)
 {
-    if (saveIfModified())
+    if (saveIfModified() && checkin(true))
     {
-        if (!systemFile && rcs && rcs->isCheckedOut())
-        {
-            if (!checkin(true))
-            {
-                ev->ignore();
-                return;
-            }
-            delete rcs;
-        }
+        if (rcs) delete rcs;
     } else
     {
         ev->ignore();
@@ -1465,10 +1548,10 @@ void FWWindow::closeEvent( QCloseEvent * ev)
 
 void FWWindow::compile()
 {
-    doCompile();
+    doCompile( tr("Close") );
 }
 
-int FWWindow::doCompile()
+int FWWindow::doCompile(const QString &closeButtonText)
 {
     if (visibleFirewall==NULL) return -1;
 
@@ -1493,7 +1576,7 @@ int FWWindow::doCompile()
             this,"Firewall Builder", 
             tr("Firewall platform is not specified in this object.\n\
 Can't compile firewall policy."),
-            "&Continue", QString::null,QString::null,
+            tr("&Continue"), QString::null,QString::null,
             0, 1 );
         return -1;
     }
@@ -1567,7 +1650,7 @@ Can't compile firewall policy."),
 
     args.push_back( fw->getName().c_str() );
 
-    execDialog dlg(this, args );
+    execDialog dlg(this, args, closeButtonText );
 
     int res= dlg.run();
 
@@ -1611,7 +1694,7 @@ void FWWindow::install()
     {
 /* need to recompile */
         fileSave();
-        if ( doCompile()!=0 ) return;
+        if ( doCompile( tr("Install") )!=0 ) return;
     } else
     {
 /* check if the time of the last modification in the database is newer
@@ -1637,7 +1720,7 @@ void FWWindow::install()
 
                 case 0:  // recompile
                     fileSave();
-                    if ( doCompile()!=0 ) return;
+                    if ( doCompile( tr("Install") )!=0 ) return;
                     break;
                 case 1:  // install old one
                     break;
@@ -1657,6 +1740,7 @@ void FWWindow::install()
 
         cnf.user          = fwopt->getStr("admUser").c_str();
         cnf.maddr         = fwopt->getStr("altAddress").c_str();
+        cnf.sshArgs       = fwopt->getStr("sshArgs").c_str();
         cnf.activationCmd = fwopt->getStr("activationCmd").c_str();
 
         cnf.fwobj     = fw;
