@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindow.h,v 1.48 2004/09/13 01:15:48 vkurland Exp $
+  $Id: FWWindow.h,v 1.52 2005/04/17 21:58:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -43,6 +43,7 @@ namespace libfwbuilder {
 class QTabWidget;
 class RuleSetView;
 class QTimer;
+class QPrinter;
 
 class FWWindow : public FWBMainWindow_q {
 
@@ -60,7 +61,9 @@ class FWWindow : public FWBMainWindow_q {
     libfwbuilder::FWObjectDatabase         *objdb;
 
     QTimer                                 *autosaveTimer;
-
+    QString                                 noFirewalls;
+    QPrinter                               *printer;
+    
     void clearFirewallTabs();
 
  public slots:
@@ -138,7 +141,7 @@ class FWWindow : public FWBMainWindow_q {
     void setFileName(const QString &fname);
     
     bool saveIfModified();
-
+    
     void updateFirewallName(libfwbuilder::FWObject *obj,const QString &oldName);
     void updateRuleSetView();
     void updateRuleOptions();
@@ -158,7 +161,7 @@ class FWWindow : public FWBMainWindow_q {
     libfwbuilder::FWObject* getVisibleFirewall() { return visibleFirewall; }
 
     void info(libfwbuilder::FWObject *o);
-    int  doCompile();
+    int  doCompile(const QString &closeButtonText="");
 
     void setupAutoSave();
     
