@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IPv4Dialog.cpp,v 1.23 2004/09/12 23:55:16 vkurland Exp $
+  $Id: IPv4Dialog.cpp,v 1.24 2004/12/09 08:33:20 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -274,3 +274,10 @@ void IPv4Dialog::DNSlookup()
     }
 }
 
+void IPv4Dialog::closeEvent(QCloseEvent *e)
+{
+    if (fwbdebug)
+        qDebug("IPv4Dialog::closeEvent  got close event: %p",e);
+    emit close_sign(e);
+}
+
