@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: LibraryDialog.cpp,v 1.22 2004/06/16 06:58:42 vkurland Exp $
+  $Id: LibraryDialog.cpp,v 1.26 2004/12/18 03:03:31 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -46,6 +46,16 @@
 using namespace std;
 using namespace libfwbuilder;
 
+LibraryDialog::LibraryDialog(QWidget *parent) : LibraryDialog_q(parent)
+{
+    obj=NULL;
+
+    WFlags flags = getWFlags();
+    flags &= ~Qt::WStyle_Maximize;
+    flags &= ~Qt::WStyle_Minimize;
+    setWFlags(flags);
+}
+
 void LibraryDialog::loadFWObject(FWObject *o)
 {
     obj=o;
@@ -100,6 +110,7 @@ void LibraryDialog::applyChanges()
     if (!validateName(this,obj,obj_name->text())) return;
 
     string oldname=obj->getName();
+    QString oldcolor=obj->getStr("color").c_str();
     obj->setName( string(obj_name->text().utf8()) );
     obj->setComment( string(comment->text().utf8()) );
     obj->setStr("color", color.latin1());
@@ -112,6 +123,7 @@ void LibraryDialog::applyChanges()
 
     om->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
     om->updateLibName(obj);
+    if (color!=oldcolor) om->updateLibColor(obj);
 
     apply->setEnabled( false );
 }
@@ -137,12 +149,12 @@ void LibraryDialog::changeColor()
 
     QColor clr = QColorDialog::getColor(
         QColor(color), this, tr("Pick the color for this library") );
-    
+    if (!clr.isValid()) return;
+
     color = clr.name();
-    obj->setStr("color", color.latin1() );
     fillColor();
 
-    om->updateLibColor(obj);
+    changed();
 }
 
 void LibraryDialog::fillColor()
