@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectManipulator.h,v 1.32 2004/08/24 04:41:21 vkurland Exp $
+  $Id: ObjectManipulator.h,v 1.36 2005/04/17 21:58:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -89,17 +89,6 @@ class ObjectManipulator : public ObjectManipulator_q {
     std::map<libfwbuilder::FWObject*, ObjectTreeViewItem*> allItems;
 
     
-/* Under some circumstances, user may select several host or fw
- * objects so that their children objects are selected as well
- * (e.g. when shift-click is used). "Delete objects" or "group
- * objects" operations will work on all children objects, which leads
- * to unexpected results since it is not obvious to the user that
- * children objects were selected (since they are invisible). We need
- * to remove them from the list before we delete or perform other
- * actions.
- */
-    void simplifySelection(std::vector<libfwbuilder::FWObject*> &selectedObjects);
-    
     ObjectTreeViewItem* insertObject( ObjectTreeViewItem *itm,libfwbuilder::FWObject *obj );
     void insertSubtree( ObjectTreeViewItem *itm,libfwbuilder::FWObject *obj );
 
@@ -152,6 +141,7 @@ class ObjectManipulator : public ObjectManipulator_q {
      void newInterface();
      void newNetwork();
      void newAddress();
+     void newInterfaceAddress();
      void newPhysicalAddress();
      void newAddressRange();
      void newObjectGroup();
@@ -241,7 +231,10 @@ class ObjectManipulator : public ObjectManipulator_q {
      */
     void closeSystemLib();
 
-    libfwbuilder::FWObject* pasteTo(libfwbuilder::FWObject *target,libfwbuilder::FWObject *obj,bool openobj=true);
+    libfwbuilder::FWObject* pasteTo(libfwbuilder::FWObject *target,
+                                    libfwbuilder::FWObject *obj,
+                                    bool openobj=true,
+                                    bool validateOnly=false);
     void delObj(libfwbuilder::FWObject *obj,bool openobj=true);
 
     /**
@@ -265,6 +258,19 @@ class ObjectManipulator : public ObjectManipulator_q {
      */
     void showDeletedObjects(bool f);
 
+
+    /**
+     * get boolean flags that describe state of the menu items.
+     * Can be used for both pop-up context menu and the main menu.
+     */ 
+    void getMenuState(bool haveMoveTargets,
+                      bool &dupMenuItem,
+                      bool &moveMenuItem,
+                      bool &copyMenuItem,
+                      bool &pasteMenuItem,
+                      bool &delMenuItem,
+                      bool &newMenuItem,
+                      bool &inDeletedObjects);
  signals:
 /**
  * the dialog class should have a slot that can load object's data
