@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: PrefsDialog.cpp,v 1.19 2004/07/11 06:43:10 vkurland Exp $
+  $Id: PrefsDialog.cpp,v 1.25 2004/12/17 08:04:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -44,6 +44,10 @@
 #include <qcolor.h>
 #include <qtooltip.h>
 #include <qspinbox.h>
+#include <qpainter.h>
+#include <qpushbutton.h>
+#include <qcolordialog.h>
+#include <qcolor.h>
 
 /*
 
@@ -65,6 +69,15 @@
 
 using namespace std;
 
+void PrefsDialog::setButtonColor(QPushButton *btn,const QString &colorCode)
+{
+    QPixmap pm(40,14);
+    pm.fill( QColor(colorCode) );
+    QPainter p( &pm );
+    p.drawRect( pm.rect() );
+    btn->setPixmap(pm);
+}
+
 PrefsDialog::PrefsDialog(QWidget *parent) : prefsDialog_q(parent)
 {
     wDir->setText( st->getWDir() );
@@ -91,60 +104,108 @@ PrefsDialog::PrefsDialog(QWidget *parent) : prefsDialog_q(parent)
     for (list<libData>::iterator i=addOnLibs->begin(); i!=addOnLibs->end(); ++i)
     {
         QListViewItem *itm = new QListViewItem( avLibs,
-                                                i->name.c_str(),
+                                      QString::fromUtf8(i->name),
                                                 "",
-                                                i->path.c_str() );
+                                                i->path );
         if (i->load)
             itm->setPixmap(1, QPixmap::fromMimeSource( "apply.png" ) );
     }
 
 // set label icons colors and text strings using user's settings
 
-    QString c,t;
-    c=st->getLabelColor(FWBSettings::RED);
+    QString t;
+    colors[FWBSettings::RED]=st->getLabelColor(FWBSettings::RED);
     t=st->getLabelText (FWBSettings::RED);
 
-    redPixmap->setPaletteBackgroundColor( QColor(c) );
+    setButtonColor(redBtn,colors[FWBSettings::RED]);
     redText->setText(t);
 
-    c=st->getLabelColor(FWBSettings::ORANGE);
+    colors[FWBSettings::ORANGE]=st->getLabelColor(FWBSettings::ORANGE);
     t=st->getLabelText (FWBSettings::ORANGE);
 
-    orangePixmap->setPaletteBackgroundColor( QColor(c) );
+    setButtonColor(orangeBtn,colors[FWBSettings::ORANGE]);
     orangeText->setText(t);
 
-    c=st->getLabelColor(FWBSettings::YELLOW);
+    colors[FWBSettings::YELLOW]=st->getLabelColor(FWBSettings::YELLOW);
     t=st->getLabelText (FWBSettings::YELLOW);
 
-    yellowPixmap->setPaletteBackgroundColor( QColor(c) );
+    setButtonColor(yellowBtn,colors[FWBSettings::YELLOW]);
     yellowText->setText(t);
 
-    c=st->getLabelColor(FWBSettings::GREEN);
+    colors[FWBSettings::GREEN]=st->getLabelColor(FWBSettings::GREEN);
     t=st->getLabelText (FWBSettings::GREEN);
 
-    greenPixmap->setPaletteBackgroundColor( QColor(c) );
+    setButtonColor(greenBtn,colors[FWBSettings::GREEN]);
     greenText->setText(t);
 
-    c=st->getLabelColor(FWBSettings::BLUE);
+    colors[FWBSettings::BLUE]=st->getLabelColor(FWBSettings::BLUE);
     t=st->getLabelText (FWBSettings::BLUE);
 
-    bluePixmap->setPaletteBackgroundColor( QColor(c) );
+    setButtonColor(blueBtn,colors[FWBSettings::BLUE]);
     blueText->setText(t);
 
-    c=st->getLabelColor(FWBSettings::PURPLE);
+    colors[FWBSettings::PURPLE]=st->getLabelColor(FWBSettings::PURPLE);
     t=st->getLabelText (FWBSettings::PURPLE);
 
-    purplePixmap->setPaletteBackgroundColor( QColor(c) );
+    setButtonColor(purpleBtn,colors[FWBSettings::PURPLE]);
     purpleText->setText(t);
 
-    c=st->getLabelColor(FWBSettings::GRAY);
+    colors[FWBSettings::GRAY]=st->getLabelColor(FWBSettings::GRAY);
     t=st->getLabelText (FWBSettings::GRAY);
 
-    grayPixmap->setPaletteBackgroundColor( QColor(c) );
+    setButtonColor(grayBtn,colors[FWBSettings::GRAY]);
     grayText->setText(t);
 
 }
 
+void PrefsDialog::changeColor(QPushButton *btn,
+                              FWBSettings::LabelColors colorCode)
+{
+    QColor clr = QColorDialog::getColor(
+        QColor(colors[colorCode]), this, tr("Pick the color") );
+
+    if (!clr.isValid()) return;
+    colors[colorCode]= clr.name();
+    setButtonColor(btn,colors[colorCode]);
+}
+
+
+void PrefsDialog::changeRedColor()
+{
+    changeColor(redBtn, FWBSettings::RED);
+}
+
+void PrefsDialog::changeOrangeColor()
+{
+    changeColor(orangeBtn, FWBSettings::ORANGE);
+}
+
+void PrefsDialog::changeYellowColor()
+{
+    changeColor(yellowBtn, FWBSettings::YELLOW);
+}
+
+void PrefsDialog::changeGreenColor()
+{
+    changeColor(greenBtn, FWBSettings::GREEN);
+}
+
+void PrefsDialog::changeBlueColor()
+{
+    changeColor(blueBtn, FWBSettings::BLUE);
+}
+
+void PrefsDialog::changePurpleColor()
+{
+    changeColor(purpleBtn, FWBSettings::PURPLE);
+}
+
+void PrefsDialog::changeGrayColor()
+{
+    changeColor(grayBtn, FWBSettings::GRAY);
+}
+
+
 void PrefsDialog::findWDir()
 {
     QString wd = st->getWDir();
@@ -174,14 +235,14 @@ void PrefsDialog::findSSH()
 
 void PrefsDialog::libClick(QListViewItem* itm, const QPoint &p, int col)
 {
-    if (col==1)
+    if (itm && col==1)
     {
         if (itm->text(0)=="Standard") return;
 
         for (list<libData>::iterator i=addOnLibs->begin();
              i!=addOnLibs->end(); ++i)
         {
-            if (QString(i->name.c_str()) == itm->text(0))
+            if (QString::fromUtf8(i->name) == itm->text(0))
             {
                 if (i->load)  itm->setPixmap(1, NULL);
                 else  itm->setPixmap(1, QPixmap::fromMimeSource( "apply.png" ) );
@@ -190,7 +251,7 @@ void PrefsDialog::libClick(QListViewItem* itm, const QPoint &p, int col)
 
                 if (i->load)
                 {
-                    mw->loadLibrary( i->path );
+                    mw->loadLibrary( i->path.latin1() );
                     om->loadObjects();
                 }
                 break;
@@ -209,11 +270,17 @@ void PrefsDialog::addLibrary()
     if (!fp.isEmpty())
     {
         list<libData>::iterator i = addOnLibs->add( fp.latin1(), true );
-        QString libname = i->name.c_str();
+        if (i==addOnLibs->end())
+        {
+            if (fwbdebug)
+                qDebug("PrefsDialog::addLibrary(): library addition failed");
+            return;
+        }
+        QString libname = QString::fromUtf8(i->name);
         QListViewItem *itm = new QListViewItem( avLibs, libname, "", fp );
         itm->setPixmap(1, QPixmap::fromMimeSource( "apply.png" ) );
 
-        mw->loadLibrary( i->path );
+        mw->loadLibrary( i->path.latin1() );
         om->loadObjects();
     }
 }
@@ -231,7 +298,7 @@ void PrefsDialog::remLibrary()
 
     for (list<libData>::iterator i=addOnLibs->begin(); i!=addOnLibs->end(); ++i)
     {
-        if (QString(i->name.c_str()) == itm->text(0))
+        if (QString::fromUtf8(i->name) == itm->text(0))
         {
             addOnLibs->erase(i);
             delete itm;
@@ -266,6 +333,14 @@ void PrefsDialog::accept()
 
     st->setDontSaveStdLib( dontSaveStdLib->isChecked() );
 
+    st->setLabelColor(FWBSettings::RED,    colors[FWBSettings::RED]);
+    st->setLabelColor(FWBSettings::ORANGE, colors[FWBSettings::ORANGE]);
+    st->setLabelColor(FWBSettings::YELLOW, colors[FWBSettings::YELLOW]);
+    st->setLabelColor(FWBSettings::GREEN,  colors[FWBSettings::GREEN]);
+    st->setLabelColor(FWBSettings::BLUE,   colors[FWBSettings::BLUE]);
+    st->setLabelColor(FWBSettings::PURPLE, colors[FWBSettings::PURPLE]);
+    st->setLabelColor(FWBSettings::GRAY,   colors[FWBSettings::GRAY]);
+
     st->setLabelText (FWBSettings::RED,    redText->text() );
     st->setLabelText (FWBSettings::ORANGE, orangeText->text() );
     st->setLabelText (FWBSettings::YELLOW, yellowText->text() );
