@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland vadim@fwbuilder.org
 
-  $Id: RCS.cpp,v 1.36 2004/08/05 05:32:44 vkurland Exp $
+  $Id: RCS.cpp,v 1.49 2005/02/12 07:57:06 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -37,6 +37,9 @@
 #include <qdir.h>
 #include <qregexp.h>
 #include <qmessagebox.h>
+#include <qapplication.h>
+#include <qeventloop.h>
+#include <qtextcodec.h>
 
 #include <stdlib.h>
 
@@ -213,8 +216,9 @@ RCS::RCS(const QString &file)
     ro            = false;
     temp          = false;
 
+    ciproc = new QProcess();
     proc = new QProcess();
-    proc->setCommunication( QProcess::Stdout|QProcess::Stderr);
+    proc->setCommunication( QProcess::Stdin|QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
 
     connect(proc, SIGNAL(readyReadStdout()), this,  SLOT(readFromStdout() ) );
     connect(proc, SIGNAL(readyReadStderr()), this,  SLOT(readFromStderr() ) );
@@ -226,6 +230,10 @@ RCS::RCS(const QString &file)
         QDir     rcsdir;
         rcsdir.cd(rcspath);
 
+/*
+ * rlog is started with environment defined by RCSEnvFix, which does
+ * not have env. var LANG so it always runs in english
+ */
         QStringList rcslog=rlog();
 
         QStringList::iterator i;
@@ -238,12 +246,14 @@ RCS::RCS(const QString &file)
                 continue;
             }
 
-            if ( (*i).find("==========")==0 ) break;
-            if ( (*i).find("----------")==0 )
+            if ( (*i).find(QRegExp("^=========="))==0 ) break;
+
+            if ((*i).find(QRegExp("^revision\\s+[0-9\\.]+"))==0)
             {
+                if (fwbdebug) qDebug("revision '%s'",(*i).ascii());
+
                 Revision r(filename);
 
-                ++i;
                 r.rev       = (*i).section(QRegExp("\\s+"),1,1);
                 QString lb  = (*i).section(QRegExp("[\\s;]+"),4,4);
                 if (lb!="")
@@ -253,21 +263,27 @@ RCS::RCS(const QString &file)
                     locked_by   = lb;
                     locked_rev  = r.rev;
                 }
-                ++i;
-                r.date      = (*i).section(QRegExp("[\\s;]+"),1,2);
-                r.author    = QString("    ")+(*i).section(QRegExp("[\\s;]+"),4,4);
-                ++i;
-                if ( (*i).find("branches: ")==0 ) ++i;
-                r.log="";
-                do {
-                    r.log= r.log + *i;
-                    ++i;
-                } while ( i!=rcslog.end() &&
-                          (*i).find("----------")!=0 &&
-                          (*i).find("==========")!=0 );
-                --i;
 
+                r.log="";
+
+                for ( ; (*i).find(QRegExp("^=========="))==-1 &&
+                          (*i).find(QRegExp("^----------"))==-1 &&
+                          i!=rcslog.end(); ++i )
+                {
+                    if ((*i).find(QRegExp("^date:.*author:.*state:"))==0)
+                    {
+                        r.date      = (*i).section(QRegExp("[\\s;]+"),1,2);
+                        r.author    = QString("    ")+(*i).section(QRegExp("[\\s;]+"),4,4);
+                        continue;
+                    }
+                    if ((*i).find(QRegExp("^branches:"))==0) continue;
+                    r.log= r.log + *i + "\n";
+                }
+                r.log.replace('\r',"");
                 revisions.push_back(r);
+                if (fwbdebug) qDebug("revision %s: '%s'",r.rev.ascii(),r.log.ascii());
+
+                if (i==rcslog.end()) break;
             }
         }
 //        qBubbleSort( revisions.begin() , revisions.end() );
@@ -640,21 +656,29 @@ after the program crashed.").arg(locked_rev),
 }
 
 
-bool RCS::ci( const QString &logmsg,
+bool RCS::ci( const QString &_lm,
               bool unlock) throw(libfwbuilder::FWException)
 {
 /* first check if filename is already in RCS */
     if (!isInRCS()) return false;
 
-    proc->clearArguments();
+    QString logmsg = _lm;
 
-    proc->addArgument( ci_file_name );
-    proc->addArgument( "-q" );
-    if (unlock) proc->addArgument( "-u" );
-    else        proc->addArgument( "-l" );
-    if (logmsg=="") proc->addArgument( QString("-m.") );
-    else            proc->addArgument( QString("-m")+logmsg );
-    proc->addArgument( filename );
+    if (logmsg.isEmpty()) logmsg="_";  // otherwise ci adds "*** empty log message ***"
+
+    if (fwbdebug)
+        qDebug("RCS::ci  log message (%d characters): '%s'",
+               logmsg.length(), logmsg.ascii());
+
+    ciproc->setCommunication( QProcess::Stdin|QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
+
+    ciproc->clearArguments();
+
+    ciproc->addArgument( ci_file_name );
+    if (unlock) ciproc->addArgument( "-u" );
+    else        ciproc->addArgument( "-l" );
+
+    ciproc->addArgument( filename );
 
     stdoutBuffer="";
     stderrBuffer="";
@@ -662,34 +686,87 @@ bool RCS::ci( const QString &logmsg,
     if (fwbdebug) qDebug("starting ci with environment '%s'",
                          rcsenvfix->getEnv()->join("\n").ascii());
 
-    if (proc->start( rcsenvfix->getEnv() ) )
+    QCString rcslog = logmsg.utf8();
+
+    QString obuf;
+
+    connect(ciproc, SIGNAL(processExited()), this,  SLOT(completeCI() ) );
+    connect(ciproc, SIGNAL(wroteToStdin()),  this,  SLOT(completeCI() ) );
+
+/*
+ * under some circumstances, ci may exit immediately (e.g. when there
+ * were no changes done to the file and it won't expect any rcs log
+ * record on stdin). In this case slot completeCI is called
+ * immediately, even before we have a chance to enter event loop. We
+ * need to make sure we do not enter event loop if this happens. We
+ * use flag ciRunning to check for that.
+ *
+ * Also it seems on windows all data is sent to the process and slot
+ * is called while we still are inside launch, so that once we exit
+ * from it, all is done and there is no need to enter event loop.
+ */    
+    ciRunning=true;
+    if (!ciproc->start(rcsenvfix->getEnv() ) )
     {
-        if (fwbdebug) qDebug("ci started");
+        if (fwbdebug) qDebug("Checkin error: file=%s error=%s",
+                             filename.latin1(),obuf.latin1());
 
-        while (proc->isRunning()) ; // cxx_sleep(1);
-
-        if (fwbdebug) qDebug("ci exited");
-
-        if (proc->normalExit() && proc->exitStatus()==0)
-        {
-            if (fwbdebug) qDebug("ci exited normally");
-            if (unlock)
-            {
-                checked_out = false;
-                locked      = false;
-            }
-            return true;
-        }
+        throw( FWException( (obuf+"\n"+
+                             ciproc->arguments().join(" ")+"\n"+
+                             rcsenvfix->getEnv()->join("\n")).ascii() ) );
     }
-    if (fwbdebug) qDebug("Checkin error: file=%s error=%s",
-                         filename.latin1(),stderrBuffer.latin1());
 
-    throw( FWException( (stdoutBuffer+"\n"+
-                         stderrBuffer+"\n"+
-                         proc->arguments().join(" ")+"\n"+
-                         rcsenvfix->getEnv()->join("\n")).ascii() ) );
+/* make a copy, omitting trailing '\0' so it won't get sent to ci */
+    QByteArray rcslogCopy;
+    rcslogCopy.duplicate((const char*)(rcslog),rcslog.length());
 
-    return false;
+    ciproc->writeToStdin(rcslogCopy);
+    ciproc->writeToStdin("\n.\n");
+
+    if (fwbdebug) qDebug("launch succeeded, entering event loop");
+
+    if (ciRunning)
+        QApplication::eventLoop()->enterLoop();
+
+    if (fwbdebug) qDebug("all data sent to ci");
+    do
+    {
+        cxx_sleep(1);
+        QString s=QString(ciproc->readStdout());
+        obuf = obuf + s;
+        if (fwbdebug) qDebug("stdoutBuffer='%s'",obuf.ascii());
+    } while (ciproc->isRunning());
+
+    if (fwbdebug) qDebug("all data collected from stdout/stderr");
+
+    while (ciproc->isRunning())
+        cxx_sleep(1);
+
+    if (fwbdebug) qDebug("ci exited");
+
+    if (ciproc->normalExit() && ciproc->exitStatus()==0)
+    {
+        if (fwbdebug) qDebug("ci exited normally");
+        if (unlock)
+        {
+            checked_out = false;
+            locked      = false;
+        }
+        return true;
+    }
+        
+    return true;
+}
+
+void RCS::completeCI()
+{
+    ciRunning=false;
+
+    int loopLevel = QApplication::eventLoop()->loopLevel();
+
+    if (fwbdebug) qDebug("completeCI: loop level %d",loopLevel);
+
+    if (loopLevel>1) QApplication::eventLoop()->exitLoop();
 }
 
 QStringList RCS::rlog() throw(libfwbuilder::FWException)
@@ -706,14 +783,41 @@ QStringList RCS::rlog() throw(libfwbuilder::FWException)
     if (!proc->start( rcsenvfix->getEnv() ) )
         throw(FWException("Fatal error running rlog "));
 
+    if (fwbdebug) qDebug("Running rlog");
+
     while (proc->isRunning()) 
     {
 	QByteArray ba = proc->readStdout();
-	if (ba.size()!=0)  stdoutBuffer=stdoutBuffer + QString(ba);
+	if (ba.size()!=0)
+        {
+            /* I've discovered that sometimes log records contain zero
+             * characters; this may be because of incorrect encoding
+             * while log records were created.
+             */
+            unsigned int pos=0;
+            int pp=-1;
+            while (pos<ba.size() && (pp=ba.find('\0',pos))!=-1)
+            {
+                pos=pp;
+                ba.at(pos)='\n';
+                ++pos;
+            }
+
+            ba.at(ba.size()-1)='\0';
+
+            stdoutBuffer=stdoutBuffer + QString(ba);
+
+            if (fwbdebug)
+                qDebug("******** Reading rlog: bufsize=%d, %s",
+                       ba.size(),(const char*)(QCString(ba)));
+
+        }
     }
 
+    QString rlogTxt = QString::fromUtf8(stdoutBuffer.ascii());
+
     if (proc->normalExit() && proc->exitStatus()==0)
-        return QStringList::split("\n",stdoutBuffer );
+        return QStringList::split("\n",rlogTxt );
     QString msg=QObject::tr("Fatal error running rlog for %1").arg(filename);
     throw( FWException( msg.latin1() ) );
 }
