@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleOptionsDialog.cpp,v 1.6 2004/08/26 05:18:32 vkurland Exp $
+  $Id: RuleOptionsDialog.cpp,v 1.7 2005/01/31 06:10:07 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -38,6 +38,7 @@
 #include "fwbuilder/Rule.h"
 #include "fwbuilder/FWOptions.h"
 #include "fwbuilder/Resources.h"
+#include "fwbuilder/Rule.h"
 
 #include <qwidgetstack.h>
 #include <qlineedit.h>
@@ -45,6 +46,8 @@
 #include <qpushbutton.h>
 #include <qspinbox.h>
 #include <qcheckbox.h>
+#include <qmessagebox.h>
+#include <qregexp.h>
 
 #include <iostream>
 
@@ -96,12 +99,18 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
         data.registerOption( ipt_logPrefix            , ropt,  "log_prefix" );
         data.registerOption( ipt_logLevel             , ropt,  "log_level" );
         data.registerOption( ipt_nlgroup              , ropt,  "ulog_nlgroup" );
+        data.registerOption( ipt_rule_name_accounting , ropt,  "rule_name_accounting" );
         data.registerOption( ipt_actionOnReject       , ropt,  "action_on_reject");
         data.registerOption( ipt_limit                , ropt,  "limit_value" );
         data.registerOption( ipt_limitSuffix          , ropt,  "limit_suffix" );
         data.registerOption( ipt_burst                , ropt,  "limit_burst" );
         data.registerOption( ipt_assumeFwIsPartOfAny  , ropt,  "firewall_is_part_of_any_and_networks" );
         data.registerOption( ipt_stateless            , ropt,  "stateless" );
+
+        bool f=false;
+        if (PolicyRule::isA(rule))
+            f = (PolicyRule::cast(rule)->getAction()==PolicyRule::Accounting);
+        ipt_rule_name_accounting->setEnabled(f);
     }
 
 
@@ -191,6 +200,24 @@ void RuleOptionsDialog::applyChanges()
 {
     if (!isTreeReadWrite(this,obj)) return;
 
+    if (platform=="iptables")
+    {
+        QString rn = ipt_rule_name_accounting->text();
+/* rule name for accounting may contain only alphanumeric characters
+ * and no white spaces or spec. characters
+ */
+        if (rn.contains(QRegExp("[^a-zA-Z0-9_]"))!=0)
+        {
+            QMessageBox::information(
+                this,"Firewall Builder", 
+                tr("Rule name for accounting is converted to the iptables\nchain name and therefore may not contain white space\nand special characters."),
+                tr("&Continue"), QString::null,QString::null,
+                0, 1 );
+
+            return;
+        }
+    }
+
     init=true;
     data.saveAll();
     init=false;
