@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleSetView.cpp,v 1.85 2004/07/25 05:51:55 vkurland Exp $
+  $Id: RuleSetView.cpp,v 1.94 2005/02/08 06:19:51 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -135,7 +135,7 @@ void RuleObjToolTip::maybeTip(const QPoint &pos)
 }
 
 
-RuleSetView::RuleSetView( int r, int c ) : QTable( r, c ), hme(this)
+RuleSetView::RuleSetView( int r, int c, QWidget *parent ) : QTable( r, c, parent ), hme(this)
 {
     kbdGoingUp = false;
     RuleElementSpacing=4;
@@ -191,6 +191,9 @@ RuleSetView::RuleSetView( int r, int c ) : QTable( r, c ), hme(this)
     connect( this, SIGNAL( selectionChanged() ),
              this, SLOT( selectionChanged() ) );
 
+    connect( this, SIGNAL( contentsMoving(int,int) ),
+             this, SLOT( contentsMoving(int,int) ) );
+
     verticalHeader()->installEventFilter( &hme );
 }
 
@@ -198,6 +201,21 @@ RuleSetView::~RuleSetView()
 {
 }
 
+void RuleSetView::contentsMoving(int x, int y)
+{
+    if (fwbdebug) qDebug("RuleSetView::contentsMoving  x=%d y=%d",x,y);
+}
+
+void RuleSetView::drawContents(QPainter *p, int cx, int cy, int cw, int ch)
+{
+    if (fwbdebug)
+        qDebug("RuleSetView::drawContents  cx=%d cy=%d cw=%d ch=%d",cx,cy,cw,ch);
+
+//    assert(ch<2000);
+
+    QTable::drawContents(p,cx,cy,cw,ch);
+}
+
 void RuleSetView::hideEvent(QHideEvent *ev)
 {
     QString k = settingsKey();
@@ -416,6 +434,7 @@ QRect RuleSetView::calculateCellSize( int row, int col )
             rule->add(nre);
         }
     }
+/* continue in Object */
     case Object:
     {
         RuleElement *re = getRE(rule,col);
@@ -435,12 +454,20 @@ QRect RuleSetView::calculateCellSize( int row, int col )
             for (FWObject::iterator j=re->begin(); j!=re->end(); j++)
             {
                 FWObject *o1= *j;
+                FWObject *o2 = o1;
+                string o1ref = "";
                 if (FWReference::cast(o1)!=NULL)
-                    o1=FWReference::cast(o1)->getPointer();
-
+                {
+                    o1ref = FWReference::cast(o1)->getPointerId();
+                    o2=FWReference::cast(o1)->getPointer();
+                }
+#if 0
+                if (fwbdebug)
+                    qDebug("+++ o1ref: %s  o2: %p",o1ref.c_str(),o2);
+#endif
                 QRect br=p.boundingRect(0, 0, 1000, 1000,
                                         Qt::AlignLeft|Qt::AlignVCenter,
-                                        QString::fromUtf8(o1->getName().c_str()) );
+                                        QString::fromUtf8(o2->getName().c_str()));
                 hc += item_h;
                 int itmW = RuleElementSpacing/2 + pixmap_w + 
                     RuleElementSpacing + br.width();
@@ -542,7 +569,8 @@ QString RuleSetView::objectText(RuleElement *re,FWObject *obj)
         QString lbl= Interface::cast(obj)->getLabel().c_str();
         if ( !lbl.isEmpty() ) return lbl;
     }
-    return QString::fromUtf8(obj->getName().c_str());
+    if (obj->getName() == "Any") return QString(tr("Any"));
+    else return QString::fromUtf8(obj->getName().c_str());
 }
 
 void RuleSetView::paintCell(QPainter *pntr,
@@ -556,6 +584,8 @@ void RuleSetView::paintCell(QPainter *pntr,
  * happens if this method is called to redraw the table when we call
  * setNumRows
  */
+//    if (fwbdebug) qDebug("Draw cell: row=%d",row);
+
     if (ruleIndex.count(row)==0) return;
 
     if (dirtyRows[row]!=0) 
@@ -569,12 +599,11 @@ void RuleSetView::paintCell(QPainter *pntr,
 
     QString     rclr;
     Rule *rule = Rule::cast( ruleIndex[row] );
-    if (rule!=NULL)
-    {
-        FWOptions  *ropt = rule->getOptionsObject();
-        assert(ropt!=NULL);
-        rclr = ropt->getStr("color").c_str();
-    }
+    if (rule==NULL) return;
+    
+    FWOptions  *ropt = rule->getOptionsObject();
+    assert(ropt!=NULL);
+    rclr = ropt->getStr("color").c_str();
 
     QPixmap bufferpixmap;
     bufferpixmap.resize( cr.width() , cr.height() );
@@ -587,6 +616,9 @@ void RuleSetView::paintCell(QPainter *pntr,
     int x  = r.left() + RuleElementSpacing/2;
     int y  = r.top();
 
+//    if (fwbdebug)
+//        qDebug("Draw cell: row=%d x=%d y=%d",row,x,y);
+
 //    QColor penColor;
 
 //    p.fillRect( 0 , 0 , cr.width()-1, cr.height()-1, cg.base());
@@ -654,7 +686,7 @@ void RuleSetView::paintCell(QPainter *pntr,
         string icn = Resources::global_res->getResourceStr("/FWBuilderResources/UI/Icons/"+ act );
         assert(icn!="");
         QPixmap pm = QPixmap::fromMimeSource( icn.c_str() );
-        p.drawPixmap( x,y,pm );
+        p.drawPixmap( x,y + RuleElementSpacing/2,pm );
         x+=pm.width()+1;
         QRect br=p.boundingRect(x, y, 1000, 1000,
                                  Qt::AlignLeft|Qt::AlignVCenter,
@@ -672,7 +704,7 @@ void RuleSetView::paintCell(QPainter *pntr,
         string icn = Resources::global_res->getResourceStr("/FWBuilderResources/UI/Icons/"+ dir );
         assert(icn!="");
         QPixmap pm = QPixmap::fromMimeSource( icn.c_str() );
-        p.drawPixmap( x,y,pm );
+        p.drawPixmap( x,y + RuleElementSpacing/2,pm );
         x+=pm.width()+1;
         QRect br=p.boundingRect(x, y, 1000, 1000,
                                  Qt::AlignLeft|Qt::AlignVCenter,dir.c_str() );
@@ -690,7 +722,7 @@ void RuleSetView::paintCell(QPainter *pntr,
             string icn = Resources::global_res->getResourceStr("/FWBuilderResources/UI/Icons/Log" );
             assert(icn!="");
             QPixmap pm = QPixmap::fromMimeSource( icn.c_str() );
-            p.drawPixmap( x,y,pm );
+            p.drawPixmap( x,y + RuleElementSpacing/2,pm );
             x+=pm.width()+2;
         }
         if (! isDefaultOptions( rule->getOptionsObject() ))
@@ -698,7 +730,7 @@ void RuleSetView::paintCell(QPainter *pntr,
             string icn = Resources::global_res->getResourceStr("/FWBuilderResources/UI/Icons/Options" );
             assert(icn!="");
             QPixmap pm = QPixmap::fromMimeSource( icn.c_str() );
-            p.drawPixmap( x,y,pm );            
+            p.drawPixmap( x,y + RuleElementSpacing/2,pm );            
         }
         break;
     }
@@ -711,7 +743,9 @@ void RuleSetView::paintCell(QPainter *pntr,
         QRect br=p.boundingRect(x, y, 1000, 1000,
                                  Qt::AlignLeft|Qt::AlignVCenter,
                                 QString::fromUtf8(rule->getComment().c_str()) );
-        p.drawText( x, y, br.width(), br.height(),
+        p.drawText( x, y + RuleElementSpacing/2,
+                    br.width(),
+                    br.height(),
 		    Qt::AlignLeft|Qt::AlignVCenter,
                     QString::fromUtf8(rule->getComment().c_str()) );
 
@@ -831,13 +865,14 @@ void RuleSetView::headerMousePressEvent(const QPoint &pos)
 
 void RuleSetView::selectionChanged()
 {
+/*
     if (fwbdebug)
     {
         qDebug("RuleSetView::selectionChanged()");
         qDebug("changingSelection   =%d",changingSelection);
         qDebug("numSelections       =%d",numSelections());
     }
-
+*/
     if (changingSelection || numSelections()==0) return;
 
     int firstSelectedRule=-1;
@@ -885,6 +920,12 @@ void RuleSetView::selectionChanged()
         qDebug("            rightCol=%d",sel.rightCol());
     }
 
+/*
+ * this fixes bug #1030538 "incorrect highlighting when selecting
+ * multiple rules"
+ */
+    updateHeaderStates();
+
     if (sel.isActive())
     {
         firstSelectedRule=sel.topRow();
@@ -892,6 +933,11 @@ void RuleSetView::selectionChanged()
 
         int selectionSize=lastSelectedRule-firstSelectedRule+1;
 
+        mw->editCopyAction->setEnabled(true);
+        mw->editCutAction->setEnabled(true);
+        mw->editPasteAction->setEnabled(true);
+        mw->editDeleteAction->setEnabled(true);
+
         mw->copyRuleAction->setEnabled( selectionSize==1 );
         mw->cutRuleAction->setEnabled( selectionSize==1 );
         mw->pasteRuleAboveAction->setEnabled( selectionSize==1 );
@@ -928,6 +974,7 @@ void RuleSetView::contextMenu(int row, int col, const QPoint &pos)
             sel= QTableSelection(row,0,row,ncols-1);
             addSelection( sel );
             setCurrentCell(row,0);
+            updateHeaderStates();
         }
     } else
     {
@@ -935,6 +982,7 @@ void RuleSetView::contextMenu(int row, int col, const QPoint &pos)
         sel= QTableSelection(row,0,row,ncols-1);
         addSelection( sel );
         setCurrentCell(row,0);
+        updateHeaderStates();
     }
 
     firstSelectedRule=sel.topRow();
@@ -981,26 +1029,56 @@ void RuleSetView::contextMenu(int row, int col, const QPoint &pos)
     {
     case Action:
     {
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Accept",     "Accept",     SLOT( changeActionToAccept()     ) );
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Deny",       "Deny",       SLOT( changeActionToDeny()       ) );
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Reject",     "Reject",     SLOT( changeActionToReject()     ) );
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Accounting", "Accounting", SLOT( changeActionToAccounting() ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Accept",
+                          tr("Accept"),
+                          SLOT( changeActionToAccept()     ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Deny",
+                          tr("Deny"),
+                          SLOT( changeActionToDeny()       ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Reject",
+                          tr("Reject"),
+                          SLOT( changeActionToReject()     ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Accounting",
+                          tr("Accounting"),
+                          SLOT( changeActionToAccounting() ) );
 
         break;
     }
     case Direction:
     {
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Inbound",  "Inbound",    SLOT( changeDirectionToIn()      ) );
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Outbound", "Outbound",   SLOT( changeDirectionToOut()     ) );
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Both",     "Both",       SLOT( changeDirectionToBoth()    ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Inbound",
+                          tr("Inbound"),
+                          SLOT( changeDirectionToIn()      ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Outbound",
+                          tr("Outbound"),
+                          SLOT( changeDirectionToOut()     ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Both",
+                          tr("Both"),
+                          SLOT( changeDirectionToBoth()    ) );
 
         break;
     }
     case Options:
     {
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Options","Rule Options", SLOT( editRuleOptions()          ) );
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Log",    "Logging On",   SLOT( changeLogToOn()            ) );
-        addPopupMenuItem( this, popup, "/FWBuilderResources/UI/Icons/Blank",  "Logging Off",   SLOT( changeLogToOff()          ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Options",
+                          tr("Rule Options"),
+                          SLOT( editRuleOptions()          ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Log",
+                          tr("Logging On"),
+                          SLOT( changeLogToOn()            ) );
+        addPopupMenuItem( this, popup,
+                          "/FWBuilderResources/UI/Icons/Blank",
+                          tr("Logging Off"),
+                          SLOT( changeLogToOff()          ) );
 
         break;
     }
@@ -1858,8 +1936,8 @@ void RuleSetView::insertRule()
     {
         firstSelectedRule=sel.topRow();
         lastSelectedRule=sel.bottomRow();
-//        removeSelection(0);
-//        verticalHeader()->update();
+        removeSelection(0);
+        verticalHeader()->update();
     } else
     {
         firstSelectedRule=currentRow();
@@ -1882,8 +1960,8 @@ void RuleSetView::addRuleAfterCurrent()
     {
         firstSelectedRule=sel.topRow();
         lastSelectedRule=sel.bottomRow();
-//        removeSelection(0);
-//        verticalHeader()->update();
+        removeSelection(0);
+        verticalHeader()->update();
     } else
     {
         firstSelectedRule=currentRow();
@@ -2191,14 +2269,25 @@ void RuleSetView::pasteRuleAbove()
     {
         firstSelectedRule=sel.topRow();
         lastSelectedRule=sel.bottomRow();
-//        removeSelection(0);
-//        verticalHeader()->update();
+        removeSelection(0);
+        verticalHeader()->update();
     } else
     {
         firstSelectedRule=currentRow();
         lastSelectedRule=currentRow();
     }
 
+/* pick rules in reverse order */
+    vector<FWObject*>::reverse_iterator i;
+    for (i= FWObjectClipboard::obj_clipboard->rbegin();
+         i!=FWObjectClipboard::obj_clipboard->rend(); ++i)
+    {
+        FWObject *co= *i;
+        if (!Rule::cast(co)) continue;
+        insertRule( firstSelectedRule, co);
+    }
+
+#if 0
     int n=0;
     vector<FWObject*>::iterator i;
     for (i= FWObjectClipboard::obj_clipboard->begin();
@@ -2208,9 +2297,7 @@ void RuleSetView::pasteRuleAbove()
         if (!Rule::cast(co)) continue;
         insertRule( firstSelectedRule+n, co);
     }
-
-//    if (FWObjectClipboard::obj_clipboard->getObject()!=NULL)
-//        insertRule( rn, Rule::cast(FWObjectClipboard::obj_clipboard->getObject()) );
+#endif
 }
 
 void RuleSetView::pasteRuleBelow()
@@ -2225,8 +2312,8 @@ void RuleSetView::pasteRuleBelow()
     {
         firstSelectedRule=sel.topRow();
         lastSelectedRule=sel.bottomRow();
-//        removeSelection(0);
-//        verticalHeader()->update();
+        removeSelection(0);
+        verticalHeader()->update();
     } else
     {
         firstSelectedRule=currentRow();
@@ -2316,7 +2403,7 @@ void RuleSetView::disableRule()
 
 
 
-PolicyView::PolicyView(Policy *p) : RuleSetView(1, 5)
+PolicyView::PolicyView(Policy *p, QWidget *parent) : RuleSetView(1, 5, parent)
 {
     setName("PolicyView");
     ruleset=p;
@@ -2398,7 +2485,8 @@ RuleElement* PolicyView::getRE( Rule* r, int col )
     return RuleElement::cast( r->getFirstByType(ret) );
 }
 
-InterfacePolicyView::InterfacePolicyView(InterfacePolicy *p) : RuleSetView(1,7)
+InterfacePolicyView::InterfacePolicyView(InterfacePolicy *p, QWidget *parent) :
+    RuleSetView(1,7,parent)
 {
     setName("InterfacePolicyView");
     ruleset=p;
@@ -2487,7 +2575,7 @@ RuleElement* InterfacePolicyView::getRE( Rule *r, int col )
 
 
 
-NATView::NATView(NAT *p) : RuleSetView(1,7)
+NATView::NATView(NAT *p, QWidget *parent) : RuleSetView(1,7,parent)
 {
     setName("NATView");
     ruleset=p;
