@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHPIX.cpp,v 1.5 2004/09/29 06:31:52 vkurland Exp $
+  $Id: SSHPIX.cpp,v 1.10 2004/11/24 06:46:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -40,6 +40,11 @@
 
 #include <iostream>
 
+#include <errno.h>
+#ifndef errno
+extern int errno;
+#endif 
+
 using namespace std;
 
 SSHPIX::SSHPIX(const QString &_h,
@@ -143,10 +148,8 @@ void SSHPIX::stateMachine()
         {
             state=LOGGEDIN;
             emit printStdout_sign( "\n");
-            emit printStdout_sign( tr("\nLogged in") );
-            emit printStdout_sign( "\n");
-            emit printStdout_sign( tr("Switching to enable mode..."));
-            emit printStdout_sign( "\n");
+            emit printStdout_sign( tr("Logged in\n") );
+            emit printStdout_sign( tr("Switching to enable mode...\n"));
             stdoutBuffer="";
             proc->writeToStdin( "enable\n" );
         }
@@ -203,7 +206,6 @@ void SSHPIX::stateMachine()
         }
         if ( cmpPrompt(stdoutBuffer,enable_prompt) )
         {       
-            emit printStdout_sign( "\n");
             emit printStdout_sign( tr("In enable mode."));
             emit printStdout_sign( "\n");
             state=ENABLE;  // and go to ENABLE target in switch
@@ -275,6 +277,13 @@ void SSHPIX::stateMachine()
                         s.stripWhiteSpace();
                         allConfig.push_back(s);
                         nLines++;
+/*
+ * store names of access-lists and object-groups actually used in the config
+ */
+                        if (s.find("access-list ")==0)
+                            newAcls.push_back(s.section(' ',1,1));
+                        if (s.find("object-group ")==0)
+                            newObjectGroups.push_back(s.section(' ',1,1));
                     }
                 }
                 config_file->close();
@@ -303,14 +312,7 @@ void SSHPIX::stateMachine()
         if ( cmpPrompt(stdoutBuffer,enable_prompt) )
         {
         loop1:
-            if ( allConfig.size()==0 )
-            {
-                state=EXIT_FROM_CONFIG;
-                emit printStdout_sign( tr("*** End ") );
-                proc->writeToStdin( "exit\n" );
-                break;
-
-            } else
+            if ( allConfig.size()!=0 )
             {
                 QString s;
 
@@ -345,10 +347,51 @@ void SSHPIX::stateMachine()
                     emit printStdout_sign( s+"\n" );
                     goto loop1;
                 }
+                break;
+            } else
+            {
+                /* allConfig.size()==0 */
+
+//                state=GET_ACLS;
+//                goto entry;
+
+                state=EXIT_FROM_CONFIG;
+                emit printStdout_sign( tr("End\n") );
+                proc->writeToStdin( "exit\n" );
             }
         }
         break;
 
+    case GET_ACLS:
+        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        {
+            QTimer::singleShot( 0, this, SLOT(getACLs()) );
+        }
+        break;
+
+    case GET_OG:
+        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        {
+            QTimer::singleShot( 0, this, SLOT(getObjectGroups()) );
+        }
+        break;
+
+
+    case CLEAR_ACLS:
+        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        {
+            QTimer::singleShot( 0, this, SLOT(clearACLs()) );
+        }
+        break;
+
+    case CLEAR_OG:
+        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        {
+            QTimer::singleShot( 0, this, SLOT(clearObjectGroups()) );
+        }
+        break;
+
+
     case EXIT_FROM_CONFIG:
         if ( cmpPrompt(stdoutBuffer,enable_prompt) )
         {
@@ -361,6 +404,7 @@ void SSHPIX::stateMachine()
  */
             if (!dry_run && !testRun && ncmd!=0)
             {
+                emit printStdout_sign( tr("Save configuration\n") );
                 state=SAVE_CONFIG;
                 proc->writeToStdin( "wr mem\n");
             } else
@@ -374,14 +418,33 @@ void SSHPIX::stateMachine()
     case SAVE_CONFIG:
         if ( cmpPrompt(stdoutBuffer,enable_prompt) )
         {
+            if (saveStandby && ncmd!=0)
+            {
+                emit printStdout_sign( tr("Save configuration to standby unit\n") );
+                state=SAVE_STANDBY;
+                proc->writeToStdin( "wr standby\n");
+            } else
+            {
+                emit printStdout_sign( tr("Exiting\n") );
+                state=EXIT;
+                proc->writeToStdin( "exit\n");
+            }
+        }
+        break;
+
+    case SAVE_STANDBY:
+        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        {
+            emit printStdout_sign( tr("Exiting\n") );
             state=EXIT;
             proc->writeToStdin( "exit\n");
         }
         break;
 
     case EXIT:
-        terminate();
-        state=FINISH;
+//        emit printStdout_sign( tr("Terminating session\n") );
+//        terminate();
+//        state=FINISH;
         break;
 
     case FINISH:
@@ -420,6 +483,137 @@ void SSHPIX::PIXbackup()
     proc->writeToStdin( "\n" );
 }
 
+void SSHPIX::getACLs()
+{
+    if (fwbdebug) qDebug("SSHPIX::getACLs  ");
+
+    bool sv=verbose;
+    bool sq=quiet;
+    verbose=false;
+    quiet=true;
+
+    QString     sa=cmd(proc,"show access-list");
+
+    QStringList showAcls;
+    showAcls=QStringList::split("\n",sa);
+
+    verbose=sv;
+    quiet=sq;
+
+/* if state changed to FINISH, there was an error and ssh terminated */
+    if (state==FINISH) return;
+    if (state==COMMAND_DONE)
+    {
+        for (QStringList::iterator i=showAcls.begin(); i!=showAcls.end(); i++)
+        {
+//            if (fwbdebug) qDebug("%s",(*i).ascii());
+            if ((*i).find("access-list ")==0 && (*i).find(";")==-1)
+            {
+                QString an=(*i).section(' ',1,1);
+                if (an!="cached" && currentAcls.find(an)==currentAcls.end())
+                    currentAcls.push_back(an);
+            }
+        }
+        state=GET_OG;
+    }
+
+    proc->writeToStdin( "\n" );
+}
+
+void SSHPIX::clearACLs()
+{
+    if (fwbdebug) qDebug("SSHPIX::clearACLs  ");
+
+    emit printStdout_sign( "\n");
+    emit printStdout_sign(tr("*** Clearing unused access lists"));
+    emit printStdout_sign( "\n");
+    
+    QString ca;
+        
+    while (currentAcls.size()!=0)
+    {
+        ca=currentAcls.front();
+        currentAcls.pop_front();
+        if (newAcls.find(ca)==newAcls.end())
+        {
+            if (fwbdebug) qDebug("clear access-list %s",ca.ascii());
+            cmd(proc,QString("clear access-list %1").arg(ca));
+
+/* if state changed to FINISH, there was an error and ssh terminated */
+            if (state==FINISH) return;
+        }
+    }
+
+    state=CLEAR_OG;
+    proc->writeToStdin( "\n" );
+}
+
+void SSHPIX::getObjectGroups()
+{
+    if (fwbdebug) qDebug("SSHPIX::getObjectGroups  ");
+
+    bool sv=verbose;
+    bool sq=quiet;
+    verbose=false;
+    quiet=true;
+
+    QString sog=cmd(proc,"show object-group");
+
+    QStringList showOG;
+    showOG=QStringList::split("\n",sog);
+
+    verbose=sv;
+    quiet=sq;
+
+/* if state changed to FINISH, there was an error and ssh terminated */
+    if (state==FINISH) return;
+    if (state==COMMAND_DONE)
+    {
+        for (QStringList::iterator i=showOG.begin(); i!=showOG.end(); i++)
+        {
+//            if (fwbdebug) qDebug("%s",(*i).ascii());
+            if ((*i).find("object-group ")==0)
+            {
+                QString ogn=(*i).section(' ',1,1);
+                if (currentObjectGroups.find(ogn)==currentObjectGroups.end())
+                    currentObjectGroups.push_back(ogn);
+            }
+        }
+        state=CLEAR_ACLS;
+    }
+
+    proc->writeToStdin( "\n" );
+}
+
+void SSHPIX::clearObjectGroups()
+{
+    if (fwbdebug) qDebug("SSHPIX::clearObjectGroups  ");
+
+    emit printStdout_sign( "\n");
+    emit printStdout_sign(tr("*** Clearing unused object groups"));
+    emit printStdout_sign( "\n");
+    
+    QString ca;
+        
+    while (currentObjectGroups.size()!=0)
+    {
+        ca=currentObjectGroups.front();
+        currentObjectGroups.pop_front();
+        if (newObjectGroups.find(ca)==newObjectGroups.end())
+        {
+            if (fwbdebug) qDebug("clear object-group %s",ca.ascii());
+            cmd(proc,QString("clear object-group %1").arg(ca));
+
+/* if state changed to FINISH, there was an error and ssh terminated */
+            if (state==FINISH) return;
+        }
+    }
+
+    state=EXIT_FROM_CONFIG;
+    emit printStdout_sign( tr("*** End ") );
+    proc->writeToStdin( "exit\n" );
+}
+
 void SSHPIX::PIXincrementalInstall()
 {
     QString current_config;
