@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHSession.h,v 1.3 2004/09/22 04:41:53 vkurland Exp $
+  $Id: SSHSession.h,v 1.7 2004/11/24 06:46:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -70,8 +70,13 @@ class SSHSession : public QObject {
                  WAITING_FOR_CONFIG_PROMPT,
                  CLEAR_CONFIG,
                  PUSHING_CONFIG,
+                 GET_ACLS,
+                 CLEAR_ACLS,
+                 GET_OG,
+                 CLEAR_OG,
                  EXIT_FROM_CONFIG,
                  SAVE_CONFIG,
+                 SAVE_STANDBY,
                  RUN_SCRIPT,
                  EXIT,
                  FINISH,
@@ -84,9 +89,12 @@ class SSHSession : public QObject {
     bool       verbose;
     bool       quiet;
 
+    bool        error;
+    
     bool        backup;
     bool        incremental;
     bool        dry_run;
+    bool        saveStandby;
     bool        stripComments;
     bool        testRun;
     QString     wdir;
@@ -152,6 +160,7 @@ public:
     void setBackup(bool f) { backup=f; }
     void setIncr(bool f) { incremental=f; }
     void setDryRun(bool f) { dry_run=f; }
+    void setSaveStandby(bool f) { saveStandby=f; }
     void setTestRun(bool f) { testRun=f; }
     void setStripComments(bool f) { stripComments=f; }
     void setWDir(const QString &wd) { wdir=wd; }
@@ -160,6 +169,7 @@ public:
     void setSaveDiff(bool f) { save_diff=f; }
     void setDiffPgm(const QString &v) { diff_pgm=v; }
     void setDiffFile(const QString &v) { diff_file=v; }
+    bool getErrorStatus() { return error; }
     
 public slots:
      void readFromStdout();
