@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHUnx.cpp,v 1.7 2004/09/23 15:11:54 vkurland Exp $
+  $Id: SSHUnx.cpp,v 1.8 2004/11/14 01:32:27 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -122,7 +122,7 @@ void SSHUnx::stateMachine()
             cmpPrompt(stdoutBuffer,fwb_prompt))
         {
             state=PUSHING_CONFIG;
-            if (!quiet) emit printStdout_sign( tr("\nLogged in\n") );
+            if (!quiet) emit printStdout_sign( tr("Logged in\n") );
 //            proc->writeToStdin( "\n" );
 //            stdoutBuffer="";
             goto push_files;
@@ -213,6 +213,10 @@ void SSHUnx::stateMachine()
     case FINISH:
         if ( proc->normalExit() && proc->exitStatus()==0)
         {
+            emit printStdout_sign( "\n");
+            emit printStdout_sign( tr("Done") );
+            emit printStdout_sign( "\n");
+
             delete proc;
             proc=NULL;
 
