@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: instDialog.cpp,v 1.51 2004/10/01 03:44:25 vkurland Exp $
+  $Id: instDialog.cpp,v 1.60 2005/04/10 07:16:39 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -77,6 +77,8 @@ instDialog::instDialog(instConf *_c)
     cnf=_c;
     activationCommandDone=false;
 
+    progressDisplay->setTextFormat(Qt::RichText);
+
     fwb_prompt="--**--**--";
     session = NULL;
 
@@ -95,6 +97,7 @@ instDialog::instDialog(instConf *_c)
 
     cnf->incremental=st->readBoolEntry("/FirewallBuilder2/Installer/incr"    );
     cnf->save_diff  =st->readBoolEntry("/FirewallBuilder2/Installer/savediff");
+    cnf->saveStandby=st->readBoolEntry("/FirewallBuilder2/Installer/saveStandby");
     cnf->dry_run    =st->readBoolEntry("/FirewallBuilder2/Installer/dryrun"  );
     cnf->quiet      =st->readBoolEntry("/FirewallBuilder2/Installer/quiet"   );
     cnf->verbose    =st->readBoolEntry("/FirewallBuilder2/Installer/verbose" );
@@ -117,6 +120,7 @@ instDialog::instDialog(instConf *_c)
         incr->hide();
         test->hide();
         saveDiff->hide();
+        saveStandby->hide();
         backupConfigFile->hide();
         backupConfigFileLbl->hide();
         epwd->hide();
@@ -126,6 +130,7 @@ instDialog::instDialog(instConf *_c)
         incr->show();
         test->show();
         saveDiff->show();
+        saveStandby->show();
         backupConfigFile->show();
         backupConfigFileLbl->show();
         epwd->show();
@@ -157,6 +162,7 @@ instDialog::instDialog(instConf *_c)
     test->setChecked( cnf->dry_run );
     backupConfigFile->setText( cnf->backup_file );
     saveDiff->setChecked( cnf->save_diff );
+    saveStandby->setChecked( cnf->saveStandby );
     altAddress->setText( cnf->maddr );
     quiet->setChecked( cnf->quiet );
     verbose->setChecked( cnf->verbose );
@@ -230,6 +236,7 @@ instDialog::~instDialog()
 
     st->writeEntry("/FirewallBuilder2/Installer/incr",    cnf->incremental);
     st->writeEntry("/FirewallBuilder2/Installer/savediff",cnf->save_diff  );
+    st->writeEntry("/FirewallBuilder2/Installer/saveStandby",cnf->saveStandby);
     st->writeEntry("/FirewallBuilder2/Installer/dryrun"  ,cnf->dry_run    );
     st->writeEntry("/FirewallBuilder2/Installer/quiet",   cnf->quiet      );
     st->writeEntry("/FirewallBuilder2/Installer/verbose", cnf->verbose    );
@@ -250,6 +257,7 @@ void instDialog::append(const QString &s)
 void instDialog::appendRich(const QString &s)
 {
     progressDisplay->append(s);
+//    progressDisplay->insertParagraph(s,-1);
 }
 
 QString instDialog::getUName() { return uname->text(); }
@@ -330,6 +338,7 @@ void instDialog::selected(const QString &title)
         cnf->backup_file = backupConfigFile->text();
         cnf->backup      = !cnf->backup_file.isEmpty();
         cnf->save_diff   = saveDiff->isChecked();
+        cnf->saveStandby = saveStandby->isChecked();
 	QString aaddr    = altAddress->text();
 	if (!aaddr.isEmpty())
 	{
@@ -438,7 +447,18 @@ void instDialog::selected(const QString &title)
 #ifdef _WIN32
             args.push_back(ssh);
 
-            if (ssh.find("plink.exe")!=-1) args.push_back("-ssh");
+/*
+ * putty ignores protocol and port specified in the session file if
+ * command line option -ssh is given.
+ *
+ * On the other hand,the sign of session usage is an empty user name,
+ * so we can check for that. If user name is empty, then putty will
+ * use current Windows account name to log in to the firewall and this
+ * is unlikely to work anyway. This seems to be a decent workaround.
+ */
+            if (!cnf->user.isEmpty() &&
+                ssh.find("plink.exe")!=-1) args.push_back("-ssh");
+
             args.push_back("-pw");
             args.push_back(cnf->pwd);
 #else
@@ -447,11 +467,19 @@ void instDialog::selected(const QString &title)
             args.push_back("-t");
             args.push_back("-t");
 #endif
-            args.push_back("-v");
+            if (!cnf->sshArgs.isEmpty())
+                args += QStringList::split(" ",cnf->sshArgs);
+
+            if (cnf->verbose) args.push_back("-v");
             if (!cnf->user.isEmpty())
-                args.push_back(cnf->user + "@" + cnf->maddr);
-            else
+            {
+                args.push_back("-l");
+                args.push_back(cnf->user);
                 args.push_back(cnf->maddr);
+            } else
+                args.push_back(cnf->maddr);
+
+            if (cnf->verbose) displayCommand(args);
 
             phase=1;
 
@@ -475,7 +503,12 @@ void instDialog::selected(const QString &title)
                     "activation/fwdir_test").c_str();
             } else
             {
-                s=cnf->fwobj->getOptionsObject()->getStr("firewall_dir").c_str();
+/* user_can_change_install_dir */
+                bool uccid=Resources::getTargetOptionBool(
+                    cnf->fwobj->getStr("host_OS"),"user_can_change_install_dir");
+
+                if (uccid)
+                    s=cnf->fwobj->getOptionsObject()->getStr("firewall_dir").c_str();
 
                 if (s.isEmpty()) s=Resources::getTargetOptionStr(
                     cnf->fwobj->getStr("host_OS"),
@@ -575,7 +608,9 @@ void instDialog::initiateCopy(const QString &file)
 #ifdef _WIN32
     args.push_back(ssh);
 
-    if (ssh.find("plink.exe")!=-1) args.push_back("-ssh");
+    if (!cnf->user.isEmpty() &&
+        ssh.find("plink.exe")!=-1) args.push_back("-ssh");
+
     args.push_back("-pw");
     args.push_back(cnf->pwd);
 #else
@@ -587,10 +622,16 @@ void instDialog::initiateCopy(const QString &file)
 /* do not change destination, we do chmod on it later */
 //        args.push_back( cnf->wdir+"/"+file);
 
-    QString sa;
-    if (!cnf->user.isEmpty())  sa=cnf->user + "@" + cnf->maddr;
-    else                       sa=cnf->maddr;
-    args.push_back(sa);
+    if (!cnf->sshArgs.isEmpty())
+        args += QStringList::split(" ",cnf->sshArgs);
+
+    if (!cnf->user.isEmpty())
+    {
+        args.push_back("-l");
+        args.push_back(cnf->user);
+        args.push_back(cnf->maddr);
+    } else
+        args.push_back(cnf->maddr);
 
     string optpath="activation/";
 
@@ -616,6 +657,8 @@ void instDialog::initiateCopy(const QString &file)
     appendRich( tr("\nCopying %1 -> %2\n")
                 .arg(cnf->wdir+"/"+file).arg(cnf->fwdir) );
 
+    if (cnf->verbose) displayCommand(args);
+
     phase=1;
 
     SSHSession *s= new SSHUnx(cnf->fwobj->getName().c_str(),
@@ -628,13 +671,42 @@ void instDialog::initiateCopy(const QString &file)
     runSSH(s);
 }
 
+void instDialog::displayCommand(const QStringList &args)
+{
+    QStringList a1 = args;
+
+    for (QStringList::iterator i=a1.begin(); i!=a1.end(); i++)
+    {
+        if ( (*i)=="-pw" )
+        {
+            i++;
+            *i = "XXXXXX";
+            break;
+        }
+    }
+    QString s=a1.join(" ");
+    appendRich( tr("Running command '%1'\n\n").arg(s) );
+    appendRich("\n");
+}
+
 void instDialog::continueRun()
 {
-    if (session!=NULL) delete session;
-    session=NULL;
+    if (session)
+    {
+        if (session->getErrorStatus())
+        {
+            appendRich( tr("ERROR: Terminating install sequence\n") );
+            setFinishEnabled( page(1), true );
+            return;
+        }
+
+        delete session;
+        session=NULL;
+    }
 
     if (activationCommandDone)
     {
+        appendRich( tr("Done\n") );
         setFinishEnabled( page(1), true );
         return;
     }
@@ -656,7 +728,9 @@ void instDialog::continueRun()
 
 #ifdef _WIN32
             args.push_back(ssh);
-            if (ssh.find("plink.exe")!=-1) args.push_back("-ssh");
+
+            if (!cnf->user.isEmpty() &&
+                ssh.find("plink.exe")!=-1) args.push_back("-ssh");
 
             args.push_back("-pw");
             args.push_back(cnf->pwd);
@@ -666,33 +740,31 @@ void instDialog::continueRun()
             args.push_back("-t");
             args.push_back("-t");
 #endif
+            if (!cnf->sshArgs.isEmpty())
+                args += QStringList::split(" ",cnf->sshArgs);
+
             if (cnf->verbose) args.push_back("-v");
 
+            if (!cnf->user.isEmpty())
+            {
+                args.push_back("-l");
+                args.push_back(cnf->user);
+                args.push_back(cnf->maddr);
+            } else
+                args.push_back(cnf->maddr);
+#if 0
             if (!cnf->user.isEmpty())
                 args.push_back(cnf->user + "@" + cnf->maddr);
             else
                 args.push_back(cnf->maddr);
+#endif
 
             QString cmd = getActivationCmd();
             args.push_back( cmd );
 
-            if (cnf->verbose)
-            {
-                QStringList a1 = args;
+            appendRich( tr("Activating new policy\n") );
 
-                for (QStringList::iterator i=a1.begin(); i!=a1.end(); i++)
-                {
-                    if ( (*i)=="-pw" )
-                    {
-                        i++;
-                        *i = "XXXXXX";
-                        break;
-                    }
-                }
-                QString s=a1.join(" ");
-                appendRich( tr("Running command '%1'\n\n").arg(s) );
-                appendRich("\n");
-            }
+            if (cnf->verbose) displayCommand(args);
 
             activationCommandDone=true;
 
