@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ipfAdvancedDialog.cpp,v 1.6 2004/09/28 05:21:00 vkurland Exp $
+  $Id: ipfAdvancedDialog.cpp,v 1.10 2005/04/10 07:16:39 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,9 +28,11 @@
 #include "platforms.h"
 
 #include "ipfAdvancedDialog.h"
+#include "SimpleTextEditor.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
+#include "fwbuilder/Resources.h"
 
 #include <qcheckbox.h>
 #include <qspinbox.h>
@@ -39,6 +41,7 @@
 #include <qlineedit.h>
 #include <qwidgetstack.h>
 #include <qregexp.h>
+#include <qtextview.h>
 
 
 using namespace std;
@@ -56,7 +59,16 @@ ipfAdvancedDialog::ipfAdvancedDialog(QWidget *parent,FWObject *o)
     Management *mgmt=(Firewall::cast(obj))->getManagementObject();
     assert(mgmt!=NULL);
 
+    if (fwbdebug)
+        qDebug("%s",Resources::getTargetOptionStr(
+                   obj->getStr("host_OS"),"user_can_change_install_dir").c_str());
 
+    if (!Resources::getTargetOptionBool(
+            obj->getStr("host_OS"),"user_can_change_install_dir"))
+    {
+        ipf_fw_dir->setEnabled(false);
+        fwopt->setStr("firewall_dir","");
+    }
 
     data.registerOption( ipf_log_or_block         ,fwopt, "ipf_log_or_block"    );
     data.registerOption( ipf_log_body             ,fwopt, "ipf_log_body"        );
@@ -75,12 +87,14 @@ ipfAdvancedDialog::ipfAdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption( ipf_fw_dir               ,fwopt, "firewall_dir"      );
     data.registerOption( ipf_user                 ,fwopt, "admUser"           );
     data.registerOption( altAddress               ,fwopt, "altAddress"        );
+    data.registerOption( sshArgs                  ,fwopt, "sshArgs"           );
     data.registerOption( activationCmd            ,fwopt, "activationCmd"     );
 
     data.registerOption( ipf_manage_virtual_addr  ,fwopt, "manage_virtual_addr");
     data.registerOption( ipf_configure_interfaces ,fwopt, "configure_interfaces");
     data.registerOption( ipf_debug                ,fwopt, "debug"            );
     data.registerOption( ipf_optimize             ,fwopt, "optimize"  );
+    data.registerOption( ipf_dynAddr              ,fwopt, "dynAddr"   );
 
     QStringList logLevels= getLogLevels( obj->getStr("platform").c_str() );
 
@@ -109,6 +123,11 @@ ipfAdvancedDialog::ipfAdvancedDialog(QWidget *parent,FWObject *o)
     installScript->setText(     pis->getCommand().c_str() );
     installScriptArgs->setText( pis->getArguments().c_str() );
 
+/* page "Prolog/Epilog" */
+    data.registerOption( prolog_script    , fwopt,  "prolog_script"  );
+
+    data.registerOption( epilog_script    , fwopt,  "epilog_script"  );
+
     data.loadAll();
 }
 
@@ -137,4 +156,21 @@ void ipfAdvancedDialog::reject()
     ipfAdvancedDialog_q::reject();
 }
 
+void ipfAdvancedDialog::editProlog()
+{
+    SimpleTextEditor edt( prolog_script->text(),
+                          true, tr( "Script Editor" ) );
+    if ( edt.exec() == QDialog::Accepted )
+        prolog_script->setText( edt.text() );
+}
+
+void ipfAdvancedDialog::editEpilog()
+{
+    SimpleTextEditor edt( epilog_script->text(),
+                          true, tr( "Script Editor" ) );
+    if ( edt.exec() == QDialog::Accepted )
+        epilog_script->setText( edt.text() );
+}
+
+
 
