@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: iptAdvancedDialog.cpp,v 1.17 2004/09/28 05:21:00 vkurland Exp $
+  $Id: iptAdvancedDialog.cpp,v 1.22 2005/04/10 07:16:39 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,9 +27,11 @@
 #include "global.h"
 
 #include "iptAdvancedDialog.h"
+#include "SimpleTextEditor.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
+#include "fwbuilder/Resources.h"
 
 #include <qcheckbox.h>
 #include <qspinbox.h>
@@ -38,6 +40,7 @@
 #include <qlineedit.h>
 #include <qwidgetstack.h>
 #include <qregexp.h>
+#include <qtextview.h>
 
 
 using namespace std;
@@ -135,6 +138,7 @@ iptAdvancedDialog::iptAdvancedDialog(QWidget *parent,FWObject *o)
     iptDebug->setChecked(            fwoptions->getBool("debug") );
     verifyInterfaces->setChecked(    fwoptions->getBool("verify_interfaces") );
     loadModules->setChecked(         fwoptions->getBool("load_modules") );
+    iptablesRestoreActivation->setChecked(   fwoptions->getBool("use_iptables_restore") );
 
 /*********************  data for fwbd and install script **************/
 //    FWBDManagement      *fwbdm = mgmt->getFWBDManagement();
@@ -143,7 +147,14 @@ iptAdvancedDialog::iptAdvancedDialog(QWidget *parent,FWObject *o)
     installScript->setText(     pis->getCommand().c_str() );
     installScriptArgs->setText( pis->getArguments().c_str() );
 
-    if (obj->getStr("host_OS")=="linksys")
+//    if (obj->getStr("host_OS")=="linksys")
+
+    if (fwbdebug)
+        qDebug("%s",Resources::getTargetOptionStr(
+                   obj->getStr("host_OS"),"user_can_change_install_dir").c_str());
+
+    if (!Resources::getTargetOptionBool(
+            obj->getStr("host_OS"),"user_can_change_install_dir"))
     {
         ipt_fw_dir->setEnabled(false);
         fwoptions->setStr("firewall_dir","");
@@ -152,8 +163,24 @@ iptAdvancedDialog::iptAdvancedDialog(QWidget *parent,FWObject *o)
     ipt_fw_dir->setText( fwoptions->getStr("firewall_dir").c_str() );
     ipt_user->setText( fwoptions->getStr("admUser").c_str() );
     altAddress->setText( fwoptions->getStr("altAddress").c_str() );
+    sshArgs->setText( fwoptions->getStr("sshArgs").c_str() );
     activationCmd->setText( fwoptions->getStr("activationCmd").c_str() );
 
+
+/* page "Prolog/Epilog" */
+    prolog_script->setText( fwoptions->getStr("prolog_script").c_str() );
+
+    string prolog_place= fwoptions->getStr("prolog_place");
+
+    if (prolog_place == "top")              prologPlace->setCurrentItem(0);
+    if (prolog_place == "after_interfaces") prologPlace->setCurrentItem(1);
+    if (prolog_place == "after_flush")      prologPlace->setCurrentItem(2);
+
+    epilog_script->setText( fwoptions->getStr("epilog_script").c_str() );
+
+
+
+
 #ifdef HAVE_LIBSSL
 
 //    int port=fwbdm->getPort();
@@ -262,6 +289,7 @@ void iptAdvancedDialog::accept()
     fwoptions->setBool("debug",            iptDebug->isChecked() );
     fwoptions->setBool("verify_interfaces",verifyInterfaces->isChecked() );
     fwoptions->setBool("load_modules",     loadModules->isChecked() );
+    fwoptions->setBool("use_iptables_restore", iptablesRestoreActivation->isChecked());
 
     fwoptions->setStr("compiler", compiler->text().latin1() );
     fwoptions->setStr("cmdline", compilerArgs->text().latin1()  );
@@ -293,8 +321,22 @@ void iptAdvancedDialog::accept()
     fwoptions->setStr("firewall_dir",ipt_fw_dir->text().latin1());
     fwoptions->setStr("admUser",ipt_user->text().latin1());
     fwoptions->setStr("altAddress",altAddress->text().latin1());
+    fwoptions->setStr("sshArgs",sshArgs->text().latin1());
     fwoptions->setStr("activationCmd",activationCmd->text().latin1());
 
+/* page "Prolog/Epilog" */
+    fwoptions->setStr("prolog_script", prolog_script->text().latin1() );
+
+    string prolog_place = "";
+    switch (prologPlace->currentItem())
+    {
+    case 1:  prolog_place = "after_interfaces"; break;
+    case 2:  prolog_place = "after_flush";      break;
+    default: prolog_place = "top";              break;
+    }
+    fwoptions->setStr("prolog_place",prolog_place);
+
+    fwoptions->setStr("epilog_script", epilog_script->text().latin1() );
 
 //    pis->setEnabled( mgmt_use_install_script->get_active() );
 //    fwbdm->setEnabled( mgmt_use_fwbd->get_active() );
@@ -307,4 +349,20 @@ void iptAdvancedDialog::reject()
     iptAdvancedDialog_q::reject();
 }
 
+void iptAdvancedDialog::editProlog()
+{
+    SimpleTextEditor edt( prolog_script->text(),
+                          true, tr( "Script Editor" ) );
+    if ( edt.exec() == QDialog::Accepted )
+        prolog_script->setText( edt.text() );
+}
+
+void iptAdvancedDialog::editEpilog()
+{
+    SimpleTextEditor edt( epilog_script->text(),
+                          true, tr( "Script Editor" ) );
+    if ( edt.exec() == QDialog::Accepted )
+        epilog_script->setText( edt.text() );
+}
+
 
