@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: iptAdvancedDialog.h,v 1.3 2004/03/30 06:55:07 vkurland Exp $
+  $Id: iptAdvancedDialog.h,v 1.4 2004/10/18 05:06:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -47,6 +47,9 @@ protected slots:
     virtual void accept();
     virtual void reject();
 
+    virtual void editProlog();
+    virtual void editEpilog();
+
 
 public slots:
     virtual void switchLOG_ULOG();
