@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: main.cpp,v 1.71 2004/09/22 04:41:53 vkurland Exp $
+  $Id: main.cpp,v 1.84 2005/02/14 07:37:56 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,7 +27,15 @@
 #include "config.h"
 #include "global.h"
 
-#include <getopt.h>
+#ifdef HAVE_GETOPT_H
+#  include <getopt.h>
+#else
+#  ifdef _WIN32
+#    include <getopt.h>
+#  else
+#    include <stdlib.h>
+#  endif
+#endif
 
 #include <qapplication.h>
 #include <qtextcodec.h>
@@ -105,21 +113,97 @@ using namespace std;
 static QString    filename;
 static QString    objid;
 
-QApplication      *app;
-FWWindow          *mw;
-ObjectManipulator *om;
-ObjectEditor      *oe;
-QTextEdit         *oi;
-FWBSettings       *st;
-findDialog        *fd;
-int                fwbdebug;
-int                safemode;
-bool               registered=false;
+QApplication      *app        = NULL;
+FWWindow          *mw         = NULL;
+ObjectManipulator *om         = NULL;
+ObjectEditor      *oe         = NULL;
+QTextEdit         *oi         = NULL;
+FWBSettings       *st         = NULL;
+findDialog        *fd         = NULL;
+int                fwbdebug   = 0;
+int                safemode   = 0;
+bool               registered = false;
     
 listOfLibraries  *addOnLibs;
 
+#ifndef _WIN32
+
+static struct termios save_termios;
+static int            ttysavefd = -1;
+static enum { RESET, RAW, CBREAK } ttystate = RESET;
+
+int tty_raw(int fd)
+{
+    struct termios buf;
+
+    if (tcgetattr(fd, &save_termios) < 0)
+        return -1;
+
+    buf = save_termios;
+
+    buf.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
+    buf.c_lflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
+    buf.c_lflag &= ~(CSIZE | PARENB);
+    buf.c_lflag |= CS8;
+    buf.c_lflag &= ~(OPOST);
+
+    buf.c_cc[VMIN] = 1;
+    buf.c_cc[VTIME] = 0;
+
+    if (tcsetattr(fd, TCSAFLUSH, &buf) < 0)
+        return -1;
+
+    ttystate = RAW;
+    ttysavefd = fd;
+    return 0;
+}
+
+int echo_off(int fd)
+{
+    struct stat statbuf;
+    if (fstat(fd,&statbuf)!=0) return 0;
+
+    struct termios stermios;
+    if (tcgetattr(fd, &stermios)<0)
+        return -1;
+    stermios.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
+    stermios.c_oflag &= ~(ONLCR);
+
+    if (tcsetattr(fd, TCSANOW, &stermios)<0)
+        return -1;
+
+    return 0;
+}
+
+ssize_t writen(int fd,const void *vptr, size_t n)
+{
+    size_t  nleft;
+    ssize_t nwritten;
+    const char *ptr;
+
+    ptr = (const char*)(vptr);
+    nleft = n;
+    while (nleft > 0)
+    {
+        if ( (nwritten = write(fd,ptr,nleft)) <= 0)
+            return nwritten;
+
+        nleft -= nwritten;
+        ptr += nwritten;
+    }
+    return n;
+}
 
 
+#ifndef strndup
+char* strndup(const char* s,int n)
+{
+    char *tbuf = (char*)malloc(n);
+    if (tbuf) memcpy(tbuf,s,n);
+    return tbuf;
+}
+#endif
+#endif
 
 void usage()
 {
@@ -172,7 +256,9 @@ int main( int argc, char ** argv )
         else
             if (strncmp(argv[i], "-Y", 2)==0) { scp_wrapper=true; continue; }
             else
-                arg[j]=strdup(argv[i]);
+                if (strncmp(argv[i], "-d", 2)==0) { fwbdebug++; continue; }
+                else
+                    arg[j]=strdup(argv[i]);
         j++;
     }
     arg[j]=NULL;
@@ -211,7 +297,8 @@ int main( int argc, char ** argv )
         if (ssh_wrapper)  arg[0]=strdup( sshcmd.latin1() );
         else              arg[0]=strdup( scpcmd.latin1() );
 
-//        qDebug("cmd: %s",arg[0]);
+        if (fwbdebug)
+            qDebug("cmd: %s",arg[0]);
 
 
 /* forks ssh with a pty and proxies its communication on stdin/stdout
@@ -225,100 +312,137 @@ int main( int argc, char ** argv )
         pid=forkpty(&mfd,slave_name,NULL,NULL);
         if (pid<0)
         {
-            cerr << "Fork failed" << strerror(errno) << endl;
+            qDebug("Fork failed: %s", strerror(errno));
             exit(1);
         }
         if (pid==0)
         { // child
 
 // turn echo off on stdin
-//            echo_off(STDIN_FILENO);
-
-            struct stat statbuf;
-            if (fstat(STDIN_FILENO,&statbuf)!=0) return 0;
-
-            struct termios stermios;
-            if (tcgetattr(STDIN_FILENO, &stermios)<0)
+            if (echo_off(STDIN_FILENO) < 0)
             {
-                cerr << "tcgetattr error";
-                exit(1);
-            }
-            stermios.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
-            stermios.c_oflag &= ~(ONLCR);
-
-            if (tcsetattr(STDIN_FILENO, TCSANOW, &stermios)<0)
-            {
-                cerr << "tcsetattr error";
+                qDebug("Can not turn echo off, tcgetattr or tcsetattr error");
                 exit(1);
             }
 
             execvp(arg[0],(char* const*)arg);
 
 // if we've got here there was an error
-            cerr << "Exec error: " << strerror(errno) << " " << arg[0] << endl;
+            qDebug("Exec error: %s %s",strerror(errno),arg[0]);
             exit(1);
         }
 
         FILE  *ssh=fdopen(mfd,"w+");
         setvbuf(ssh,NULL,_IONBF,0);
 
+        tty_raw(STDIN_FILENO);
+
         fd_set  rfds;
         struct timeval tv;
         int     retval;
-        char    buf[8192];
-        bool    watchstdin = true;
+
+        #define BUFFSIZE 512
+
+        char    ibuf[BUFFSIZE];
+        char    obuf[BUFFSIZE];
+        bool    endOfStream = false;
 
         while (true)
         {
-            tv.tv_sec  = 5;
+            tv.tv_sec  = 2;
             tv.tv_usec = 0;
 
             FD_ZERO(&rfds);
             FD_SET(mfd,            &rfds);
-            if (watchstdin)  FD_SET(STDIN_FILENO  , &rfds);
+            if (!endOfStream)  FD_SET(STDIN_FILENO  , &rfds);
 
             retval = select( max(STDIN_FILENO,mfd)+1 , &rfds, NULL, NULL, &tv);
+            if (retval==0)  // timeout
+            {
+                if (fwbdebug) qDebug("timeout");
+                if (endOfStream)
+                {
+                    if (fwbdebug) qDebug("Closing mfd");
+                    close(mfd);
+                    break;
+                }
+            }
             if (retval)
             {
                 if (FD_ISSET(STDIN_FILENO, &rfds))
                 {
-                    int n=read(0,buf,sizeof(buf));
+                    int n=read(0,ibuf,sizeof(ibuf));
+                    if (fwbdebug) qDebug("Read %d bytes from stdin",n);
+                    if (n<0)
+                    {
+                        if (fwbdebug) qDebug("Error on stdin");
+                        break;
+                    }
                     if (n==0)
                     {
 // eof on stdin
-                        watchstdin=false;
-                        int r=fflush(ssh);
-                        if (r) cerr << "fflush error: " << strerror(errno) << endl;
-/*
- * ugly hack, but closing mfd kills ssh before the process on the
- * other side writes all data to the file
- */
-                        sleep(2);
-                        close(mfd);
+                        if (fwbdebug) qDebug("EOF on stdin");
+                        endOfStream = true;
+//                        if (fwbdebug) qDebug("Flushing ssh");
+//                        int r=fflush(ssh);
+//                        if (r) qDebug("fflush error: %s",strerror(errno));
                     } else
-                        write(mfd,buf,n);
+                    {
+// TODO: do not forget to remove this debugging section
+                        if (fwbdebug)
+                        {
+                            int firstLF=-1;
+                            int lastLF=-1;
+                            int i=0;
+                            for (char *c=ibuf; i<n; i++,c++)
+                            {
+                                if (*c=='\n')
+                                {
+                                    lastLF=i;
+                                    if (firstLF==-1) firstLF=i;
+                                }
+                            }
+                            char *firstLine = strndup(ibuf,firstLF+1);
+                            firstLine[firstLF]='\0';
+
+                            int nn = n-lastLF-1;                           
+                            char *lastLine = "";
+                            if (nn!=0)
+                            {
+                                lastLine = strndup(ibuf+lastLF+1,nn);
+                                lastLine[nn]='\0';
+                            }
+                            qDebug("First line: %d: 0-%d  '%s'",firstLF+1,firstLF,firstLine);
+                            qDebug("Last line:  %d: %d-%d '%s'",nn,lastLF,n-1,lastLine);
+                        }
+                        int r=writen(mfd,ibuf,n);
+                        if (fwbdebug) qDebug("Wrote %d bytes to mfd",r);
+                    }
                 }
                 if (FD_ISSET(mfd, &rfds))
                 {
                     int n;
-                    buf[0]='\0';
-                    n=read(mfd,buf,sizeof(buf));
+                    obuf[0]='\0';
+                    n=read(mfd,obuf,sizeof(obuf));
+                    if (fwbdebug) qDebug("Read %d bytes from mfd",n);
                     if (n<=0)
                     {
 /* eof on mfd - this means ssh process has died */
+                        if (fwbdebug) qDebug("EOF on mfd");
                         break;
                     } else
                     {
-                        buf[n]='\0';
-//                  cerr << "read " << n << " bytes" << endl;
-                        write(1,buf,n);
+                        obuf[n]='\0';
+                        int r=writen(1,obuf,n);
+                        if (fwbdebug) qDebug("Wrote %d bytes to stdout",r);
                     }
                 }
             }
         }
         int status;
         waitpid(pid, &status, 0);
-        exit(status);
+        if (WIFEXITED(status)) exit(WEXITSTATUS(status));
+        exit(0);
     }
 #endif
 
@@ -414,10 +538,17 @@ int main( int argc, char ** argv )
 
         if (fwbdebug) qDebug("loading translation for the current locale ...");
 
-        QTranslator *translator = new QTranslator (NULL);
-        translator->load(QString("fwbuilder_")+QTextCodec::locale(), localepath.c_str() );
+        QTranslator qt1(0);
+        qt1.load( QString( "qt_" ) + QTextCodec::locale(), QTTRANSLATIONSDIR );
+        app->installTranslator( &qt1 );
 
-        app->installTranslator (translator);
+        QTranslator qt2(0);
+        qt2.load( QString( "qt_" ) + QTextCodec::locale(),  localepath.c_str() );
+        app->installTranslator( &qt2 );
+
+        QTranslator translator(0);
+        translator.load(QString("fwbuilder_")+QTextCodec::locale(),localepath.c_str());
+        app->installTranslator (&translator);
 
 #if 0
         st = new FWBSettings();
@@ -447,28 +578,46 @@ int main( int argc, char ** argv )
         {
             if (filename!="")
             {
-                RCS *rcs=new RCS(filename);
-                rcs->co();
-                mw->load(NULL,rcs);
+                try
+                {
+                    RCS *rcs=new RCS(filename);
+                    rcs->co();
+                    mw->load(NULL,rcs);
+                } catch (FWException &ex)
+                {
+                    qDebug("Exception: %s",ex.toString().c_str());
+                    mw->load(NULL);
+                }
             } else
             {
                 int sa = st->getStartupAction();
                 switch (sa) 
                 {
-                case 1:
+                case 0:   // load standard objects
+                {
+                    mw->load(NULL); // load standard objects
+                    break;
+                }
+                case 1:   // load last edited file
                 {
                     filename=st->getLastEdited();
                     if (filename!="")
                     {
-                        RCS *rcs=new RCS(filename);
-                        rcs->co();
-                        mw->load(NULL,rcs);
-
+                        try
+                        {
+                            RCS *rcs=new RCS(filename);
+                            rcs->co();
+                            mw->load(NULL,rcs);
+                        } catch (FWException &ex)
+                        {
+                            qDebug("Exception: %s",ex.toString().c_str());
+                            mw->load(NULL);
+                        }
                     } else
                         mw->load(NULL);
                     break;
                 }
-                default:
+                default:  // ask user
                 {
 /* no file specified on command line and autoload option is off */
                     StartWizard sw;
@@ -521,6 +670,9 @@ int main( int argc, char ** argv )
         {
 /* save the state of the GUI (opened firewall, opened object tree page, etc */
             FWObject *o=mw->getVisibleFirewall();
+            if (fwbdebug)
+                qDebug("Main: closing. VisibleFirewall = %p",o);
+
             if (o) st->setStr("UI/visibleFirewall", o->getId().c_str() );
 
             o=om->getOpened();
