@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newFirewallDialog.cpp,v 1.19 2004/06/16 06:58:42 vkurland Exp $
+  $Id: newFirewallDialog.cpp,v 1.23 2005/04/30 18:22:27 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -39,7 +39,6 @@
 #include "fwbuilder/Library.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Resources.h"
-#include "fwbuilder/snmp.h"
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/InterfacePolicy.h"
 #include "fwbuilder/BackgroundOp.h"
@@ -55,9 +54,13 @@
 #include <qmessagebox.h>
 #include <qtimer.h>
 #include <qlistbox.h>
+#include <qapplication.h>
+#include <qcursor.h>
 
 #include <iostream>
 
+// must be the last for win
+#include "fwbuilder/snmp.h"
 
 //#undef HAVE_LIBSNMP
 
@@ -69,6 +72,9 @@ newFirewallDialog::newFirewallDialog() : newFirewallDialog_q()
     nfw=NULL; 
     snmpPollCompleted=false;
     q=NULL;
+    unloadTemplatesLib = false;
+    getInterfacesBusy = false;
+
     timer = new QTimer(this);
     connect( timer, SIGNAL(timeout()), this, SLOT(monitor()) );
 
@@ -146,7 +152,7 @@ void newFirewallDialog::changed()
 
 void  newFirewallDialog::monitor()
 {
-    if (logger==NULL) return;
+    if (logger==NULL || q==NULL) return;
 
 #ifdef HAVE_LIBSNMP
 
@@ -183,10 +189,14 @@ void  newFirewallDialog::monitor()
                               dn,
                               idata.physicalAddress.c_str() );
 
-            cerr << "Added interface " << idata.name << endl;
+//            cerr << "Added interface " << idata.name << endl;
 
         }
     }
+
+    delete q;
+    q=NULL;
+
 #endif
 
     snmpPollCompleted=true;
@@ -197,6 +207,9 @@ void newFirewallDialog::getInterfacesViaSNMP()
 {
 #ifdef HAVE_LIBSNMP
 
+// need to protect from reentry because getAddrByName processes events
+    if (q!=NULL || getInterfacesBusy) return;
+
     snmpPollCompleted=false;
     iface_list->clear();
 
@@ -211,12 +224,16 @@ void newFirewallDialog::getInterfacesViaSNMP()
         return ;
     }
 
+    getInterfacesBusy = true;
+
     IPAddress addr;
-    string name=obj_name->text().latin1();
+    QString name=obj_name->text().latin1();
     try 
     {
-        vector<IPAddress> addrs=DNS::getHostByName( name );
-        addr= addrs.front();
+        QApplication::setOverrideCursor( QCursor( Qt::WaitCursor) );
+        QString a = getAddrByName(name);
+        QApplication::restoreOverrideCursor();
+        addr = a.ascii();
     } catch (FWException &ex)
     {
         QMessageBox::warning(
@@ -224,6 +241,7 @@ void newFirewallDialog::getInterfacesViaSNMP()
             tr("Address of %1 could not be obtained via DNS")
             .arg(obj_name->text()),
             "&Continue", QString::null, QString::null, 0, 1 );
+        getInterfacesBusy = false;
         return ;
     }
 
@@ -245,6 +263,8 @@ void newFirewallDialog::getInterfacesViaSNMP()
         //do nothing
     }
 
+    getInterfacesBusy = false;
+
 #endif
 }
 
@@ -323,6 +343,7 @@ void newFirewallDialog::selected(const QString &title)
         {
             FWObject *cl = om->getCurrentLib();
             mw->loadLibrary(tempfname);
+            unloadTemplatesLib = true;
             om->loadObjects();
             tlib = mw->db()->getById(TEMPLATE_LIB);
 /* restore library that was opened prior loading templates */
@@ -689,6 +710,29 @@ void newFirewallDialog::accept()
             itm=itm->itemBelow();
         }
     }
+    if (unloadTemplatesLib)
+    {
+        FWObject *tlib = mw->db()->getById(TEMPLATE_LIB);
+        assert (tlib!=NULL);
+        string tlibID = tlib->getId();
+        if (fwbdebug) qDebug("  Delete library of templates");
+        om->delObj(tlib,false);
+
+/*
+ * deleting an object places it in the "Deleted objects" library, so
+ * we need to remove "templates" library from there.
+ *
+ * TODO: need to add flags to the API to be able to delete objects
+ * without placing them in "Deleted objects" automatically
+ */
+        FWObject *delObjLib = mw->db()->getById( DELETED_LIB );
+        if (delObjLib!=NULL && delObjLib->getById(tlibID)!=NULL)
+        {
+            if (fwbdebug) qDebug("  Delete library of templates from 'Deleted objects'");
+            om->delObj(tlib,false);  // this time from deleted objects lib
+        }
+        unloadTemplatesLib=false;
+    }
     QDialog::accept();
 }
 
