@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pfAdvancedDialog.cpp,v 1.13 2004/09/28 05:21:00 vkurland Exp $
+  $Id: pfAdvancedDialog.cpp,v 1.16 2005/04/10 07:16:39 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,9 +28,11 @@
 #include "platforms.h"
 
 #include "pfAdvancedDialog.h"
+#include "SimpleTextEditor.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
+#include "fwbuilder/Resources.h"
 
 #include <qcheckbox.h>
 #include <qspinbox.h>
@@ -39,6 +41,7 @@
 #include <qlineedit.h>
 #include <qwidgetstack.h>
 #include <qregexp.h>
+#include <qtextview.h>
 
 
 using namespace std;
@@ -56,6 +59,17 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
     Management *mgmt=(Firewall::cast(obj))->getManagementObject();
     assert(mgmt!=NULL);
 
+    if (fwbdebug)
+        qDebug("%s",Resources::getTargetOptionStr(
+                   obj->getStr("host_OS"),"user_can_change_install_dir").c_str());
+
+    if (!Resources::getTargetOptionBool(
+            obj->getStr("host_OS"),"user_can_change_install_dir"))
+    {
+        pf_fw_dir->setEnabled(false);
+        fwopt->setStr("firewall_dir","");
+    }
+
     data.registerOption( pf_log_prefix          ,fwopt, "log_prefix"          );
     data.registerOption( pf_fallback_log        ,fwopt, "fallback_log"        );
     data.registerOption( pf_do_timeout_interval ,fwopt, "pf_do_timeout_interval" );
@@ -92,6 +106,7 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption( pf_fw_dir              ,fwopt, "firewall_dir"    );
     data.registerOption( pf_user                ,fwopt, "admUser"         );
     data.registerOption( altAddress             ,fwopt, "altAddress"      );
+    data.registerOption( sshArgs                ,fwopt, "sshArgs"         );
     data.registerOption( activationCmd          ,fwopt, "activationCmd"   );
 
     data.registerOption( pf_manage_virtual_addr ,fwopt, "manage_virtual_addr" );
@@ -143,6 +158,12 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
     installScript->setText(     pis->getCommand().c_str() );
     installScriptArgs->setText( pis->getArguments().c_str() );
 
+/* page "Prolog/Epilog" */
+    data.registerOption( prolog_script    , fwopt,   "prolog_script"  );
+    data.registerOption( epilog_script    , fwopt,   "epilog_script"  );
+
+
+
     data.loadAll();
 
     doScrubToggled();
@@ -213,4 +234,22 @@ void pfAdvancedDialog::reject()
     pfAdvancedDialog_q::reject();
 }
 
+void pfAdvancedDialog::editProlog()
+{
+    SimpleTextEditor edt( prolog_script->text(),
+                          true, tr( "Script Editor" ) );
+    if ( edt.exec() == QDialog::Accepted )
+        prolog_script->setText( edt.text() );
+}
+
+void pfAdvancedDialog::editEpilog()
+{
+    SimpleTextEditor edt( epilog_script->text(),
+                          true, tr( "Script Editor" ) );
+    if ( edt.exec() == QDialog::Accepted )
+        epilog_script->setText( edt.text() );
+}
+
+
+
 
