@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pixAdvancedDialog.cpp,v 1.14 2004/09/28 05:21:00 vkurland Exp $
+  $Id: pixAdvancedDialog.cpp,v 1.18 2005/04/10 07:16:39 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -47,6 +47,7 @@
 #include <qtextview.h>
 #include <qtabwidget.h>
 #include <qlistbox.h>
+#include <qlabel.h>
 
 #include <iostream>
 #include <sstream>
@@ -157,6 +158,24 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
     FWOptions *fwoptions=(Firewall::cast(obj))->getOptionsObject();
     assert(fwoptions!=NULL);
 
+    bool f1=fwoptions->getBool("pix_acl_basic");
+    bool f2=fwoptions->getBool("pix_acl_no_clear");
+    bool f3=fwoptions->getBool("pix_acl_substitution");
+    bool f4=fwoptions->getBool("pix_add_clear_statements");
+
+    /*
+     * If none of the new pix_acl_* options is set and old pix_add_clear_statements
+     * option is true, set pix_acl_basic to true.
+     * 
+     * If old option pix_add_clear_statements iss false, set
+     * pix_acl_no_clear to true
+     */
+    if (!f1 && !f2 && !f3)
+    {
+        if ( f4 ) fwoptions->setBool("pix_acl_basic",true);
+        else fwoptions->setBool("pix_acl_no_clear",true);
+    }
+
     Management *mgmt=(Firewall::cast(obj))->getManagementObject();
     assert(mgmt!=NULL);
 
@@ -173,6 +192,25 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption( pix_emulate_out_acl  , fwoptions,  
                          "pix_emulate_out_acl" );
 
+
+    data.registerOption( pix_acl_basic , fwoptions,  
+                         "pix_acl_basic" );
+
+/*
+    data.registerOption( pix_acl_alwaysNew , fwoptions,  
+                         "pix_acl_always_new" );
+*/
+
+    data.registerOption( pix_acl_no_clear , fwoptions,  
+                         "pix_acl_no_clear" );
+
+    data.registerOption( pix_acl_substitution , fwoptions,  
+                         "pix_acl_substitution" );
+
+    data.registerOption( pix_acl_temp_addr , fwoptions,  
+                         "pix_acl_temp_addr" );
+
+
     data.registerOption( pix_include_comments , fwoptions,  
                          "pix_include_comments" );
 
@@ -182,8 +220,10 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption( pix_regroup_commands , fwoptions,  
                          "pix_regroup_commands" );
 
+/*
     data.registerOption( pix_add_clear_statements , fwoptions,  
                          "pix_add_clear_statements" );
+*/
 
     data.registerOption( pix_optimize_default_nat , fwoptions,  
                          "pix_optimize_default_nat" );
@@ -214,6 +254,7 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
 
     data.registerOption( user             ,fwoptions, "admUser"         );
     data.registerOption( altAddress       ,fwoptions, "altAddress"      );
+    data.registerOption( sshArgs          ,fwoptions, "sshArgs"         );
 
     PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
 
@@ -432,6 +473,7 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
     data.loadAll();
     loadFixups();
     showFixups();
+    scriptACLModeChanged();
 }
 
 /*
@@ -769,4 +811,11 @@ void pixAdvancedDialog::regenerateFixups()
 {
 }
 
+void pixAdvancedDialog::scriptACLModeChanged()
+{
+    pix_acl_temp_lbl->setEnabled(pix_acl_substitution->isChecked());
+    pix_acl_temp_addr->setEnabled(pix_acl_substitution->isChecked());
+}
+
+
 
