@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: platforms.cpp,v 1.10 2004/09/15 05:44:09 vkurland Exp $
+  $Id: platforms.cpp,v 1.12 2005/01/31 06:10:07 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -75,6 +75,7 @@ bool isDefaultOptions(FWOptions *opt)
 	    res= ( opt->getStr("log_prefix").empty()       && 
                    opt->getStr("log_level").empty()        && 
 		   opt->getStr("action_on_reject").empty() && 
+		   opt->getStr("rule_name_accounting").empty() && 
 		   opt->getInt("limit_value")<=0  &&
 		   opt->getInt("limit_burst")<=0  &&
 		   ! opt->getBool("stateless") &&
@@ -124,8 +125,8 @@ QMap<QString,QString> getVersionsForPlatform(const QString &platform)
     if (platform=="iptables")
     {
         res[""]         = QObject::tr("- any -");
-        res["lt_1.2.6"] = "1.2.6";
-        res["ge_1.2.6"] = "1.2.6 to 1.2.8";
+        res["lt_1.2.6"] = QObject::tr("1.2.5 or earlier");
+        res["ge_1.2.6"] = QObject::tr("1.2.6 to 1.2.8");
         res["1.2.9"]    = QObject::tr("1.2.9 or later");
     } else
     {
