@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipf.cpp,v 1.12 2004/09/28 05:21:00 vkurland Exp $
+  $Id: ipf.cpp,v 1.18 2005/04/14 06:30:37 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -35,9 +35,11 @@
 #include <algorithm>
 #include <functional>
 #include <iostream>
+#include <stdexcept>
 
 #ifndef _WIN32
 #  include <unistd.h>
+#  include <pwd.h>
 #else
 #  include <direct.h>
 #  include <stdlib.h>
@@ -108,6 +110,26 @@ void usage(const char *name)
     cout << _("Usage: ") << name << " [-x] [-v] [-V] [-f filename.xml] [-o output.fw] [-d destdir] [-m] firewall_object_name" << endl;
 }
 
+string printActivationCommandWithSubstitution(Firewall *fw,string filePath,string cmd)
+{
+    ostringstream str;
+
+    str << "cat " << filePath << " | grep -v '#' ";
+
+    FWObjectTypedChildIterator j=fw->findByType(Interface::TYPENAME);
+    for ( ; j!=j.end(); ++j ) 
+    {
+        Interface *iface=Interface::cast(*j);
+        if ( iface->isDyn() )
+        {
+            str << "| sed \"s/ (" << iface->getName() << ") "
+                << "/ $i_" << iface->getName() << " /\"";
+        }
+    }
+    str << " | " << cmd << endl;
+    return str.str();
+}
+
 int main(int argc, char * const *argv)
 {   
 
@@ -248,7 +270,6 @@ int main(int argc, char * const *argv)
             Interface *iface=dynamic_cast<Interface*>(*i);
             assert(iface);
 
-            string::size_type n;
             if ( iface->isDyn())  
             {
                 list<FWObject*> l3=iface->getByType(IPv4::TYPENAME);
@@ -316,11 +337,18 @@ int main(int argc, char * const *argv)
  * Process firewall options, build OS network configuration script
  */
 	OSConfigurator *oscnf=NULL;
-	if (fw->getStr("host_OS")=="solaris")   oscnf=new OSConfigurator_solaris(FWObjectDatabase::db , fwobjectname);
-	if (fw->getStr("host_OS")=="openbsd")   oscnf=new OSConfigurator_openbsd(FWObjectDatabase::db , fwobjectname);
-	if (fw->getStr("host_OS")=="freebsd")   oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+        string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
+        if ( family=="solaris" )
+            oscnf=new OSConfigurator_solaris(FWObjectDatabase::db , fwobjectname);
+
+	if ( family=="openbsd")
+            oscnf=new OSConfigurator_openbsd(FWObjectDatabase::db , fwobjectname);
+
+	if ( family=="freebsd")
+            oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+
 	if (oscnf==NULL)
-	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS"));
+	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
 
 	oscnf->prolog();
 
@@ -351,7 +379,7 @@ int main(int argc, char * const *argv)
 	n.setDebugRule(  drn );
 	n.setVerbose( verbose );
 
-	bool     have_nat;
+	bool     have_nat=false;
 	if ( n.prolog() > 0 ) {
 
 	    have_nat=true;
@@ -376,17 +404,27 @@ int main(int argc, char * const *argv)
 #ifdef _WIN32
         char* user_name=getenv("USERNAME");
 #else
-        char* user_name=getenv("USER");
+        struct passwd *pwd=getpwuid(getuid());
+        assert(pwd);
+        char *user_name=pwd->pw_name;
 #endif
-        if (user_name==NULL) {
-            cerr << _("Can't figure out your user name, aborting") << endl;
-            exit(1);
+        if (user_name==NULL)
+        {
+            user_name=getenv("LOGNAME");
+            if (user_name==NULL)
+            {
+                cerr << _("Can't figure out your user name, aborting") << endl;
+                exit(1);
+            }
         }
 
+	ofstream fw_file;
+        fw_file.exceptions(ofstream::eofbit|ofstream::failbit|ofstream::badbit);
+
 #ifdef _WIN32
-	ofstream fw_file(fw_file_name.c_str(), ios::out|ios::binary);
+        fw_file.open(fw_file_name.c_str(), ios::out|ios::binary);
 #else
-	ofstream fw_file(fw_file_name.c_str());
+        fw_file.open(fw_file_name.c_str());
 #endif
 	fw_file << "#!/bin/sh " << shell_dbg << endl << endl;
 
@@ -422,56 +460,111 @@ int main(int argc, char * const *argv)
 
         fw_file << endl;
 
-        fw_file << "log \"";
+        fw_file << "log '";
         fw_file << _("Activating firewall script generated ")
                << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
                << user_name;
-        fw_file << "\"" << endl;
+        fw_file << "'" << endl;
 
 	fw_file << endl;
 
-
-
-
-
         fw_file << endl 
                 << "$IPF -Fa" << endl
                 << "$IPNAT -C" << endl;
-      
-	if (have_ipf) 
+
+/*
+ *  we add prolog and epilog to the activation shell script rather
+ *  than to ipf and nat .conf files. This is more flexible since user
+ *  can execute some shell commands, as well as add any policy and/or
+ *  nat rules by putting them into their .conf file and loading them
+ *  from prolog or epilog script. Because of this, prolog is added
+ *  after all policy and nat rules are flushed.
+ */ 
+        fw_file << endl;
+        fw_file << "#" << endl;
+        fw_file << "# Prolog script" << endl;
+        fw_file << "#" << endl;
+        
+        string pre_hook= fw->getOptionsObject()->getStr("prolog_script");
+        fw_file << pre_hook << endl;
+
+        fw_file << "#" << endl;
+        fw_file << "# End of prolog script" << endl;
+        fw_file << "#" << endl;
+
+      	if (have_ipf) 
         {
+            ofstream ipf_file;
+            ipf_file.exceptions(ofstream::eofbit|ofstream::failbit|ofstream::badbit);
+        
 #ifdef _WIN32
-            ofstream ipf_file(ipf_file_name.c_str(), ios::out|ios::binary);
+            ipf_file.open(ipf_file_name.c_str(), ios::out|ios::binary);
 #else
-            ofstream ipf_file(ipf_file_name.c_str());
+            ipf_file.open(ipf_file_name.c_str());
 #endif
 	    ipf_file << c.getCompiledScript();
 	    ipf_file.close();
-	    fw_file << "$IPF " << ipf_dbg << " -I -f "
-                    << "${FWDIR}/" << ipf_file_name << endl;
+
+            string cmd = string("$IPF ")+ipf_dbg+" -I -f ";
+            string filePath = string("${FWDIR}/")+ipf_file_name;
+            if (fw->getOptionsObject()->getBool("dynAddr"))
+            {
+                cmd += "-";
+                fw_file << 
+                    printActivationCommandWithSubstitution(fw,filePath,cmd);
+            } else
+            {
+                fw_file << cmd << filePath << endl;
+            }
 	}
 
 	if (have_nat) 
         {
+            ofstream nat_file;
+            nat_file.exceptions(ofstream::eofbit|ofstream::failbit|ofstream::badbit);
+
 #ifdef _WIN32
-            ofstream nat_file(nat_file_name.c_str(), ios::out|ios::binary);
+            nat_file.open(nat_file_name.c_str(), ios::out|ios::binary);
 #else
-            ofstream nat_file(nat_file_name.c_str());
+            nat_file.open(nat_file_name.c_str());
 #endif
 	    nat_file << n.getCompiledScript();
 	    nat_file.close();
-	    fw_file << "$IPNAT " << ipf_dbg << " -f "
-                    << "${FWDIR}/" << nat_file_name <<  endl;
+
+            string cmd = string("$IPNAT ")+ipf_dbg+" -f ";
+            string filePath = string("${FWDIR}/")+nat_file_name;
+            if (fw->getOptionsObject()->getBool("dynAddr"))
+            {
+                cmd += "-";
+                fw_file << 
+                    printActivationCommandWithSubstitution(fw,filePath,cmd);
+            } else
+            {
+                fw_file << cmd << filePath << endl;
+            }
 	}
 
 	if (have_ipf)
 	    fw_file << "$IPF " << ipf_dbg << " -s " << endl;
 
+        fw_file << endl;
+        fw_file << "#" << endl;
+        fw_file << "# Epilog script" << endl;
+        fw_file << "#" << endl;
+
+        string post_hook= fw->getOptionsObject()->getStr("epilog_script");
+        fw_file << post_hook << endl;
+
+        fw_file << endl;
+        fw_file << "# End of epilog script" << endl;
+        fw_file << "#" << endl;
+
 	fw_file << endl;
 
         fw_file << "/sbin/kldstat -n ipl.ko > /dev/null 2>&1 || $IPF -E" << endl;
 
 	fw_file << endl;
+        fw_file.close();
 
 #ifdef _WIN32
         _chmod(fw_file_name.c_str(),_S_IREAD|_S_IWRITE);
@@ -481,13 +574,20 @@ int main(int argc, char * const *argv)
         
         return 0;
 
-    } catch(FWException &ex)  {
+    } catch(const FWException &ex)  {
 	cerr << ex.toString() << endl;
         return 1;
-    } catch (std::string s) {
+#if __GNUC__ >= 3
+/* need to check version because std::ios::failure does not seem to be
+ * supported in gcc 2.9.5 on FreeBSD 4.10 */
+    } catch (const std::ios::failure &e) {
+        cerr << "Error while opening or writing to the output file" << endl;
+        return 1;
+#endif
+    } catch (const std::string &s) {
 	cerr << s;
         return 1;
-    } catch (std::exception ex) {
+    } catch (const std::exception &ex) {
 	cerr << ex.what();
         return 1;
     } catch (...) {
