@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipfw.cpp,v 1.10 2004/09/28 05:21:00 vkurland Exp $
+  $Id: ipfw.cpp,v 1.15 2005/04/14 06:30:37 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -35,9 +35,11 @@
 #include <algorithm>
 #include <functional>
 #include <iostream>
+#include <stdexcept>
 
 #ifndef _WIN32
 #  include <unistd.h>
+#  include <pwd.h>
 #else
 #  include <direct.h>
 #  include <stdlib.h>
@@ -237,7 +239,6 @@ int main(int argc, char * const *argv)
             Interface *iface=dynamic_cast<Interface*>(*i);
             assert(iface);
 
-            string::size_type n;
             if ( iface->isDyn())  
             {
                 list<FWObject*> l3=iface->getByType(IPv4::TYPENAME);
@@ -303,10 +304,15 @@ int main(int argc, char * const *argv)
  * Process firewall options, build OS network configuration script
  */
 	OSConfigurator *oscnf=NULL;
-	if (fw->getStr("host_OS")=="macosx")    oscnf=new OSConfigurator_macosx(FWObjectDatabase::db , fwobjectname);
-	if (fw->getStr("host_OS")=="freebsd")   oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+        string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
+	if ( family=="macosx")
+            oscnf=new OSConfigurator_macosx(FWObjectDatabase::db , fwobjectname);
+
+	if ( family=="freebsd")
+            oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+
 	if (oscnf==NULL)
-	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS"));
+	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
 
 	oscnf->prolog();
 /*
@@ -344,17 +350,27 @@ int main(int argc, char * const *argv)
 #ifdef _WIN32
         char* user_name=getenv("USERNAME");
 #else
-        char* user_name=getenv("USER");
+        struct passwd *pwd=getpwuid(getuid());
+        assert(pwd);
+        char *user_name=pwd->pw_name;
 #endif
-        if (user_name==NULL) {
-            cerr << _("Can't figure out your user name, aborting") << endl;
-            exit(1);
+        if (user_name==NULL)
+        {
+            user_name=getenv("LOGNAME");
+            if (user_name==NULL)
+            {
+                cerr << _("Can't figure out your user name, aborting") << endl;
+                exit(1);
+            }
         }
 
+	ofstream fw_file;
+        fw_file.exceptions(ofstream::eofbit|ofstream::failbit|ofstream::badbit);
+
 #ifdef _WIN32
-	ofstream fw_file(fw_file_name.c_str(), ios::out|ios::binary);
+        fw_file.open(fw_file_name.c_str(), ios::out|ios::binary);
 #else
-	ofstream fw_file(fw_file_name.c_str());
+        fw_file.open(fw_file_name.c_str());
 #endif
 	fw_file << "#!/bin/sh " << shell_dbg << endl << endl;
 
@@ -383,15 +399,27 @@ int main(int argc, char * const *argv)
 
         fw_file << "cd " << firewall_dir << " || exit 1" << endl << endl;
 
+        fw_file << endl;
+        fw_file << "#" << endl;
+        fw_file << "# Prolog script" << endl;
+        fw_file << "#" << endl;
+
+        string pre_hook= fw->getOptionsObject()->getStr("prolog_script");
+        fw_file << pre_hook << endl;
+
+        fw_file << "#" << endl;
+        fw_file << "# End of prolog script" << endl;
+        fw_file << "#" << endl;
+
 	fw_file << oscnf->getCompiledScript();
 
         fw_file << endl;
 
-        fw_file << "log \"";
+        fw_file << "log '";
         fw_file << _("Activating firewall script generated ")
                << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
                << user_name;
-        fw_file << "\"" << endl;
+        fw_file << "'" << endl;
 
 	fw_file << endl;
 
@@ -406,7 +434,20 @@ int main(int argc, char * const *argv)
 	    fw_file << c.getCompiledScript();
         }
 
+        fw_file << endl;
+        fw_file << "#" << endl;
+        fw_file << "# Epilog script" << endl;
+        fw_file << "#" << endl;
+
+        string post_hook= fw->getOptionsObject()->getStr("epilog_script");
+        fw_file << post_hook << endl;
+
+        fw_file << endl;
+        fw_file << "# End of epilog script" << endl;
+        fw_file << "#" << endl;
+
 	fw_file << endl;
+	fw_file.close();
 
 #ifdef _WIN32
         _chmod(fw_file_name.c_str(),_S_IREAD|_S_IWRITE);
@@ -416,13 +457,20 @@ int main(int argc, char * const *argv)
         
         return 0;
 
-    } catch(FWException &ex)  {
+    } catch(const FWException &ex)  {
 	cerr << ex.toString() << endl;
         return 1;
-    } catch (std::string s) {
+#if __GNUC__ >= 3
+/* need to check version because std::ios::failure does not seem to be
+ * supported in gcc 2.9.5 on FreeBSD 4.10 */
+    } catch (const std::ios::failure &e) {
+        cerr << "Error while opening or writing to the output file" << endl;
+        return 1;
+#endif
+    } catch (const std::string &s) {
 	cerr << s;
         return 1;
-    } catch (std::exception ex) {
+    } catch (const std::exception &ex) {
 	cerr << ex.what();
         return 1;
     } catch (...) {
