@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.cpp,v 1.13 2004/09/13 02:40:02 vkurland Exp $
+  $Id: PolicyCompiler_ipt.cpp,v 1.24 2005/04/14 06:30:37 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -69,6 +69,17 @@ static std::map<std::string,int> tmp_chain_no;
 
 string PolicyCompiler_ipt::myPlatformName() { return "iptables"; }
 
+string PolicyCompiler_ipt::getInterfaceVarName(FWObject *iface)
+{
+    ostringstream  ostr;
+    string iname=iface->getName();
+    string::size_type p1;
+    while ( (p1=iname.find("."))!=string::npos)
+        iname=iname.replace(p1,1,"_");
+    ostr << "i_" << iname;
+    return ostr.str();
+}
+
 string PolicyCompiler_ipt::getNewTmpChainName(PolicyRule *rule)
 {
     std::ostringstream str;
@@ -329,6 +340,9 @@ int PolicyCompiler_ipt::prolog()
 
 void PolicyCompiler_ipt::addPredefinedPolicyRules()
 {
+#if 0
+/* This is huge overkill. Just print the thing in ipt.cpp */
+
     FWOptions *fwopt    = getCachedFwOpt();
 
     if ( fwopt->getBool("mgmt_ssh") && ! fwopt->getStr("mgmt_addr").empty() )
@@ -395,7 +409,7 @@ void PolicyCompiler_ipt::addPredefinedPolicyRules()
 
         FWOptions  *ruleopt = r->getOptionsObject();
         ruleopt->setBool("firewall_is_part_of_any_and_networks",true);
-        ruleopt->setStr("log_prefix","INVALID state -- %A");
+        ruleopt->setStr("log_prefix","INVALID state -- %A ");
 
         RuleElement *srv=RuleElement::cast(
             r->getFirstByType(RuleElementSrv::TYPENAME) );
@@ -403,7 +417,7 @@ void PolicyCompiler_ipt::addPredefinedPolicyRules()
 
 	combined_ruleset->push_front(r);
     }
-
+#endif
 
 }
 
@@ -1064,8 +1078,9 @@ bool PolicyCompiler_ipt::bridgingFw::checkForMatchingBroadcastAndMulticast(Addre
 
  */
                 IPNetwork n( ipv4->getAddress() , ipv4->getNetmask() );
+                if (n.getAddress()==obj1_addr)  return true; 
                 if (n.getBroadcastAddress()==obj1_addr)  return true;
-            }
+           }
         }
     }
     return false;
@@ -1080,16 +1095,32 @@ bool PolicyCompiler_ipt::bridgingFw::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
-    Address        *src=compiler->getFirstSrc(rule);
+//    Address        *src=compiler->getFirstSrc(rule);
     Address        *dst=compiler->getFirstDst(rule);
 
 
-    if ( rule->getStr("ipt_chain")=="OUTPUT" || rule->getStr("ipt_chain")=="INPUT" )
+    if ( rule->getStr("ipt_chain")=="INPUT" )
     {
-
-        if (checkForMatchingBroadcastAndMulticast(src) ||
-            checkForMatchingBroadcastAndMulticast(dst) )
-            rule->setStr("ipt_chain","FORWARD");
+        if ( checkForMatchingBroadcastAndMulticast(dst) )
+        {
+/* bug #1101910: "Samba problem with Bridged Firewall"
+ * need to split rule to take care of broadcasts forwarded by the bridge, as well 
+ * as broadcasts that are accepted by the firewall itself. Need to do this only if
+ * the rule is not associated with any bridging interfaces
+ */
+            Interface *rule_iface = compiler->getCachedFwInterface(rule->getInterfaceId());
+            if (rule_iface!=NULL && rule_iface->isUnnumbered())
+                rule->setStr("ipt_chain","FORWARD");
+            else
+            {
+                PolicyRule *r= PolicyRule::cast(
+                    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+                compiler->temp_ruleset->add(r);
+                r->duplicate(rule);
+                r->setStr("ipt_chain","FORWARD");
+                tmp_queue.push_back(r);
+            }
+        }
     }
 
 
@@ -1126,6 +1157,10 @@ bool PolicyCompiler_ipt::splitIfSrcNegAndFw::processNext()
    "normal" cases are handled by splitIfSrcAny
 
  */
+
+    list<FWObject*> fwLikes;
+    list<FWObject*> notFwLikes;
+
     if (rule->getDirection()!=PolicyRule::Inbound && srcrel->getNeg())
     {
         for (list<FWObject*>::iterator i1=srcrel->begin(); i1!=srcrel->end(); ++i1) 
@@ -1135,25 +1170,36 @@ bool PolicyCompiler_ipt::splitIfSrcNegAndFw::processNext()
 
             Address *a = Address::cast(o);
             if (a && compiler->complexMatch(a,compiler->fw))
-            {
-                PolicyRule *r= PolicyRule::cast(
-                    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
-                compiler->temp_ruleset->add(r);
-                r->duplicate(rule);
-                r->setStr("ipt_chain","OUTPUT");
-                r->setDirection( PolicyRule::Outbound );
-                nsrc=r->getSrc();  nsrc->clearChildren();
-                nsrc->addRef(o);
-//                r->setLogging(false);
-                tmp_queue.push_back(r);
+                fwLikes.push_back(o);
+            else
+                notFwLikes.push_back(o);
+        }
 
-                rule->setStr("ipt_chain","FORWARD");
+        if (fwLikes.size() != 0)
+        {
+            PolicyRule *r= PolicyRule::cast(
+                compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            r->setStr("ipt_chain","OUTPUT");
+            r->setDirection( PolicyRule::Outbound );
+            nsrc=r->getSrc();
+            nsrc->clearChildren();
+            for (list<FWObject*>::iterator m=fwLikes.begin(); m!=fwLikes.end(); ++m) 
+                nsrc->addRef(*m);
+            tmp_queue.push_back(r);
 
-                srcrel->removeRef(o);
-                if (srcrel->isAny())  srcrel->setNeg(false);
-                tmp_queue.push_back(rule);
-                return true;
-            }
+            rule->setStr("ipt_chain","FORWARD");
+            nsrc=rule->getSrc();
+            nsrc->clearChildren();  // do not use reset, it clears negation flag
+            for (list<FWObject*>::iterator m=notFwLikes.begin(); m!=notFwLikes.end(); ++m) 
+                nsrc->addRef(*m);
+            /* after clearChildren RE has zero children and since it does not
+             * have 'any network' as a child, isAny() returns false
+             */
+            if (nsrc->isAny() || nsrc->size()==0) nsrc->setNeg(false);
+            tmp_queue.push_back(rule);
+            return true;
         }
     }
     tmp_queue.push_back(rule);
@@ -1189,6 +1235,10 @@ bool PolicyCompiler_ipt::splitIfDstNegAndFw::processNext()
    "normal" cases are handled by splitIfDstAny
 
  */
+
+    list<FWObject*> fwLikes;
+    list<FWObject*> notFwLikes;
+
     if (rule->getDirection()!=PolicyRule::Outbound && dstrel->getNeg())
     {
         for (list<FWObject*>::iterator i1=dstrel->begin(); i1!=dstrel->end(); ++i1) 
@@ -1198,25 +1248,34 @@ bool PolicyCompiler_ipt::splitIfDstNegAndFw::processNext()
 
             Address *a = Address::cast(o);
             if (a && compiler->complexMatch(a,compiler->fw))
-            {
-                PolicyRule *r= PolicyRule::cast(
-                    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
-                compiler->temp_ruleset->add(r);
-                r->duplicate(rule);
-                r->setStr("ipt_chain","INPUT");
-                r->setDirection( PolicyRule::Inbound );
-                ndst=r->getDst();  ndst->clearChildren();
-                ndst->addRef(o);
-//                r->setLogging(false);
-                tmp_queue.push_back(r);
+                fwLikes.push_back(o);
+            else
+                notFwLikes.push_back(o);
+        }
 
-                rule->setStr("ipt_chain","FORWARD");
+        if (fwLikes.size() != 0)
+        {
+            PolicyRule *r= PolicyRule::cast(
+                compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            r->setStr("ipt_chain","INPUT");
+            r->setDirection( PolicyRule::Inbound );
+            ndst=r->getDst();
+            ndst->clearChildren();
+            for (list<FWObject*>::iterator m=fwLikes.begin(); m!=fwLikes.end(); ++m) 
+                ndst->addRef(*m);
+            tmp_queue.push_back(r);
 
-                dstrel->removeRef(o);
-                if (dstrel->isAny())  dstrel->setNeg(false);
-                tmp_queue.push_back(rule);
-                return true;
-            }
+            rule->setStr("ipt_chain","FORWARD");
+            ndst=rule->getDst();
+            ndst->clearChildren();  // do not use reset, it clears negation flag
+            for (list<FWObject*>::iterator m=notFwLikes.begin(); m!=notFwLikes.end(); ++m) 
+                ndst->addRef(*m);
+
+            if (ndst->isAny() || ndst->size()==0) ndst->setNeg(false);
+            tmp_queue.push_back(rule);
+            return true;
         }
     }
     tmp_queue.push_back(rule);
@@ -1228,9 +1287,14 @@ bool PolicyCompiler_ipt::splitIfSrcAny::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
-    FWOptions *fwopt    = compiler->getCachedFwOpt();
+//    FWOptions *fwopt    = compiler->getCachedFwOpt();
     FWOptions  *ruleopt = rule->getOptionsObject();
-    if ( fwopt->getBool("bridging_fw") ||
+/* commented to fix bug #1112470
+ * if fw is considered part of any, we should place rule in INPUT/OUTPUT
+ * chains even if it is a bridging fw since fw itself may send or receive
+ * packets
+ */
+    if ( /* fwopt->getBool("bridging_fw") || */
          ! ruleopt->getBool("firewall_is_part_of_any_and_networks") ) 
     {
 	tmp_queue.push_back(rule);
@@ -1245,16 +1309,9 @@ bool PolicyCompiler_ipt::splitIfSrcAny::processNext()
 
     RuleElementSrc *srcrel=rule->getSrc();
     Address        *src=compiler->getFirstSrc(rule); 
-//    RuleElementDst *dstrel=rule->getDst();
-//    Address        *dst=compiler->getFirstDst(rule); 
-   
-//    bool    fw_in_dst = (dstrel->size()==1 && dst!=NULL &&
-//                         compiler->complexMatch(dst,compiler->fw));
 
     if ( rule->getDirection()!=PolicyRule::Inbound &&
          ( 
-//             (srcrel->isAny() && ! fw_in_dst) ||
-
              srcrel->isAny() ||
 
              ( srcrel->size()==1 && src!=NULL &&
@@ -1281,9 +1338,14 @@ bool PolicyCompiler_ipt::splitIfDstAny::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
-    FWOptions *fwopt = compiler->getCachedFwOpt();
+//    FWOptions *fwopt = compiler->getCachedFwOpt();
     FWOptions  *ruleopt = rule->getOptionsObject();
-    if ( fwopt->getBool("bridging_fw") ||
+/* commented to fix bug #1112470
+ * if fw is considered part of any, we should place rule in INPUT/OUTPUT
+ * chains even if it is a bridging fw since fw itself may send or receive
+ * packets
+ */
+    if ( /* fwopt->getBool("bridging_fw") || */
          ! ruleopt->getBool("firewall_is_part_of_any_and_networks") ) 
     {
 	tmp_queue.push_back(rule);
@@ -1296,18 +1358,12 @@ bool PolicyCompiler_ipt::splitIfDstAny::processNext()
 	return true;
     }
 
-    RuleElementSrc *srcrel=rule->getSrc();
-    Address        *src=compiler->getFirstSrc(rule); 
     RuleElementDst *dstrel=rule->getDst();
     Address        *dst=compiler->getFirstDst(rule); 
    
-    bool    fw_in_src = (srcrel->size()==1 && src!=NULL &&
-                         compiler->complexMatch(src,compiler->fw));
-
-
     if ( rule->getDirection()!=PolicyRule::Outbound &&
          ( 
-             ( dstrel->isAny() && !fw_in_src)  ||
+             dstrel->isAny() ||
 
              ( dstrel->size()==1 && dst!=NULL &&
                !compiler->complexMatch(dst,compiler->fw) &&
@@ -1739,7 +1795,7 @@ bool PolicyCompiler_ipt::expandLoopbackInterfaceAddress::processNext()
  *  splitIfSrcFWNetwork / splitIfDstFWNetwork and possibly split it. We will
  *  set chain to FORWARD after that in decideOnChain
  */
-bool PolicyCompiler_ipt::decideOnChainIfFW::processNext()
+bool PolicyCompiler_ipt::decideOnChainIfSrcFW::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
@@ -1749,9 +1805,79 @@ bool PolicyCompiler_ipt::decideOnChainIfFW::processNext()
         return true;
     }
 
-//    RuleElementSrc *srcrel=rule->getSrc();
     Address        *src   =compiler->getFirstSrc(rule);  assert(src);
-//    RuleElementDst *dstrel=rule->getDst();
+
+/* Bug 811860: "IPTables Compiler Firewall IP to Input Chain".
+ * on a bridging firewall rules not associated with interfaces should
+ * go into INPUT/OUTPUT chain on interfaces that do routing and into
+ * FORWARD chain on bridging interfaces. Sometimes bridging interfaces
+ * are not created in the GUI, so to play it safe we will split the
+ * rule and put it into both FORWARD and INPUT/OUTPUT chain.
+ *
+ * Bug #934949: "duplicate rules". Split the rule only if firewall is
+ * in src or dst. Otherwise compiler produces duplicates.
+ *
+ */
+    if ( compiler->getCachedFwOpt()->getBool("bridging_fw") &&
+         compiler->complexMatch(src,compiler->fw,false,false) )
+    {
+	PolicyRule *r;
+
+	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+	compiler->temp_ruleset->add(r);
+	r->duplicate(rule);
+        r->setStr("ipt_chain","FORWARD");
+
+	tmp_queue.push_back(r);
+    }
+
+    bool b,m;
+/* 
+ * do not check for broadcasts and multicasts in bridging firewall because
+ * they should go to FORWARD chain 
+ *
+ *    b=m= !( compiler->getCachedFwOpt()->getBool("bridging_fw") );
+ *
+ *
+ *  Commented out the line above while working on the bug #811860:
+ *  "IPTables Compiler Firewall IP to Input Chain".  No need to do it
+ *  anymore since we now split the rule if we work with a bridging fw
+ *  and the rule _always_ goes into FORWARD chain --vk 09/28/03
+ */
+
+    b=m=true;
+
+    switch ( rule->getDirection() ) 
+    {
+    case PolicyRule::Outbound:
+/* if direction is "Outbound", chain can never be INPUT, but could be FORWARD */
+        if (!src->isAny() && compiler->complexMatch(src,compiler->fw,b,m))
+            rule->setStr("ipt_chain","OUTPUT");
+        break;
+
+    case PolicyRule::Both:
+/* direction == Both
+ */
+        if (!src->isAny() && compiler->complexMatch(src,compiler->fw,b,m)) 
+            rule->setStr("ipt_chain","OUTPUT");
+        break;
+    default: break;
+    }
+    tmp_queue.push_back(rule);
+
+    return true;
+}
+
+bool PolicyCompiler_ipt::decideOnChainIfDstFW::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+
+    if ( ! rule->getStr("ipt_chain").empty() )
+    {
+        tmp_queue.push_back(rule);
+        return true;
+    }
+
     Address        *dst   =compiler->getFirstDst(rule);  assert(dst);
 
 /* Bug 811860: "IPTables Compiler Firewall IP to Input Chain".
@@ -1766,8 +1892,7 @@ bool PolicyCompiler_ipt::decideOnChainIfFW::processNext()
  *
  */
     if ( compiler->getCachedFwOpt()->getBool("bridging_fw") &&
-         (compiler->complexMatch(dst,compiler->fw,false,false) ||
-          compiler->complexMatch(src,compiler->fw,false,false)) )
+         compiler->complexMatch(dst,compiler->fw,false,false) )
     {
 	PolicyRule *r;
 
@@ -1803,32 +1928,22 @@ bool PolicyCompiler_ipt::decideOnChainIfFW::processNext()
             rule->setStr("ipt_chain","INPUT");
         break;
 
-    case PolicyRule::Outbound:
-/* if direction is "Outbound", chain can never be INPUT, but could be FORWARD */
-        if (!src->isAny() && compiler->complexMatch(src,compiler->fw,b,m))
-            rule->setStr("ipt_chain","OUTPUT");
-        break;
-
-    default:
-
+    case PolicyRule::Both:
 /* direction == Both
  */
         if (!dst->isAny() && compiler->complexMatch(dst,compiler->fw,b,m)) 
-        {
             rule->setStr("ipt_chain","INPUT");
-            break;
-        }
-        if (!src->isAny() && compiler->complexMatch(src,compiler->fw,b,m)) 
-        {
-            rule->setStr("ipt_chain","OUTPUT");
-            break;
-        }
+        break;
+    default: break;
     }
     tmp_queue.push_back(rule);
 
     return true;
 }
 
+
+
+
 /*
  * processor splitIfIfaceAndDirectionBoth splits interface rule if its
  * direction is "Both". This means that by the time when this
@@ -1874,9 +1989,13 @@ bool PolicyCompiler_ipt::finalizeChain::processNext()
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
 
-    tmp_queue.push_back(rule);
+//    tmp_queue.push_back(rule);
 
-    if ( ! rule->getStr("ipt_chain").empty() ) return true;
+    if ( ! rule->getStr("ipt_chain").empty() )
+    {
+        tmp_queue.push_back(rule);
+        return true;
+    }
 
     rule->setStr("ipt_chain","FORWARD");
 
@@ -1898,14 +2017,15 @@ bool PolicyCompiler_ipt::finalizeChain::processNext()
 /* if direction is "Inbound", chain can never be OUTPUT, but could be FORWARD */
         if (!dst->isAny() && ipt_comp->complexMatch(dst,ipt_comp->fw,b,m))
             rule->setStr("ipt_chain","INPUT");
-        return true;
 
+        break;
 
     case PolicyRule::Outbound:
 /* if direction is "Outbound", chain can never be INPUT, but could be FORWARD */
         if (!src->isAny() && ipt_comp->complexMatch(src,ipt_comp->fw,b,m))
             rule->setStr("ipt_chain","OUTPUT");
-	return true;
+
+        break;
 
     default:
 
@@ -1913,15 +2033,29 @@ bool PolicyCompiler_ipt::finalizeChain::processNext()
         if (!dst->isAny() && ipt_comp->complexMatch(dst,ipt_comp->fw,b,m)) 
         {
             rule->setStr("ipt_chain","INPUT");
-            return true;
+            break;
         }
         if (!src->isAny() && ipt_comp->complexMatch(src,ipt_comp->fw,b,m)) 
         {
             rule->setStr("ipt_chain","OUTPUT");
-            return true;
+            break;
         }
     }
+/*
+ * bug #1040599: "unnecessary FORWARD rules".
+ * If we haven't decided on INPUT/OUTPUT chain, it stays FORWARD.
+ * However, if ip forwarding is turned off, we do not want any rules
+ * in FORWARD chain, so we just drop it.
+ *
+ * If ip forwarding is set to "no change", assume it is on.
+ */
+    bool ipforw=true;
+    string s=compiler->getCachedFwOpt()->getStr("linux24_ip_forward");
+    if (!s.empty() && (s=="0" || s=="Off" || s=="off")) ipforw=false;
 
+    if (rule->getStr("ipt_chain")=="FORWARD" && !ipforw) return true;
+
+    tmp_queue.push_back(rule);
     return true;
 }
 
@@ -2427,39 +2561,50 @@ bool PolicyCompiler_ipt::accounting::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     Interface  *rule_iface = compiler->getCachedFwInterface(rule->getInterfaceId());
+    FWOptions  *ruleopt =rule->getOptionsObject();
 
-    if (rule->getAction()==PolicyRule::Accounting)
+    if (rule->getAction()==PolicyRule::Accounting &&
+        rule->getStr("ipt_target").empty())
     {
         string this_chain  =rule->getStr("ipt_chain");
 	string new_chain=PolicyCompiler_ipt::getNewChainName(rule,rule_iface);
+        string rule_name_accounting = ruleopt->getStr("rule_name_accounting");
+        if (!rule_name_accounting.empty())
+            new_chain = rule_name_accounting;
 
-	PolicyRule     *r;
-        FWOptions      *ruleopt;
-        RuleElementSrc *nsrc;
-        RuleElementDst *ndst;
-        RuleElementSrv *nsrv;
+        if (new_chain==this_chain)
+        {
+            rule->setStr("ipt_target","RETURN");
+            rule->setAction(PolicyRule::Continue);
+        } else
+        {
+            PolicyRule     *r;
+            FWOptions      *ruleopt;
+            RuleElementSrc *nsrc;
+            RuleElementDst *ndst;
+            RuleElementSrv *nsrv;
 /*
  * add copy of original rule, but turn off logging and set target
  * chain to new_chain.
  */
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,true) );
-	compiler->temp_ruleset->add(r);
-	r->duplicate(rule);
-	nsrc=r->getSrc();  nsrc->reset();
-	ndst=r->getDst();  ndst->reset();
-	nsrv=r->getSrv();  nsrv->reset();
-	r->setStr("ipt_chain",new_chain);
-        r->setStr("upstream_rule_chain",this_chain);
-	r->setStr("ipt_target","RETURN");
-	r->setLogging(false);
-        r->setAction(PolicyRule::Continue);
-	tmp_queue.push_back(r);
-
-	rule->setStr("ipt_target",new_chain);
-	rule->setLogging(false);
-        ruleopt =rule->getOptionsObject();
-        ruleopt->setInt("limit_value",-1);
+            r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,true) );
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            nsrc=r->getSrc();  nsrc->reset();
+            ndst=r->getDst();  ndst->reset();
+            nsrv=r->getSrv();  nsrv->reset();
+            r->setStr("ipt_chain",new_chain);
+            r->setStr("upstream_rule_chain",this_chain);
+            r->setStr("ipt_target","RETURN");
+            r->setLogging(false);
+            r->setAction(PolicyRule::Continue);
+            tmp_queue.push_back(r);
 
+            rule->setStr("ipt_target",new_chain);
+            rule->setLogging(false);
+            ruleopt =rule->getOptionsObject();
+            ruleopt->setInt("limit_value",-1);
+        }
     } 
     tmp_queue.push_back(rule);
 
@@ -2551,6 +2696,8 @@ bool PolicyCompiler_ipt::addPredefinedRules::processNext()
  */
 void PolicyCompiler_ipt::compile()
 {
+    printRule=NULL;
+
     cout << _(" Compiling policy for ") << fw->getName() << " ..." <<  endl << flush;
 
     try {
@@ -2616,23 +2763,29 @@ void PolicyCompiler_ipt::compile()
  * that rule element.
  */
 //        add( new singleSrvNegation(          "process negation in Srv if it holds single object"               ) );
-        add( new splitRuleIfSrvAnyActionReject("split rule if action is reject and srv is any" ) );
+        add( new splitRuleIfSrvAnyActionReject(
+                 "split rule if action is reject and srv is any" ) );
         add( new SrvNegation(       "process negation in Srv"               ) );
         add( new expandGroupsInSrv( "expand groups in Srv"                  ) );
-//        add( new splitRuleIfSrvAnyActionReject("split rule if action is reject and srv is any" ) );
+//        add( new splitRuleIfSrvAnyActionReject(
+//                 "split rule if action is reject and srv is any" ) );
         add( new fillActionOnReject("fill in action_on_reject"              ) );
-        add( new splitServicesIfRejectWithTCPReset("check and split if action on reject is TCP reset"));
+        add( new splitServicesIfRejectWithTCPReset(
+                 "check and split if action on reject is TCP reset"));
         add( new fillActionOnReject("fill in action_on_reject 2"            ) );
-        add( new splitServicesIfRejectWithTCPReset("check and split if action on reject is TCP reset 2"));
-        add( new singleSrcNegation( "process negation in Src if it holds single object"               ) );
-        add( new singleDstNegation( "process negation in Dst if it holds single object"               ) );
+        add( new splitServicesIfRejectWithTCPReset(
+                 "check and split if action on reject is TCP reset 2"));
+        add( new singleSrcNegation(
+                 "process negation in Src if it holds single object"        ) );
+        add( new singleDstNegation(
+                 "process negation in Dst if it holds single object"        ) );
 
 /*
  * phased out these processors, they are not needed anymore because we use variable
  * for dynamic interfaces.
  */
-        add( new splitIfSrcNegAndFw(          "split rule if src has negation and fw" ) );
-        add( new splitIfDstNegAndFw(          "split rule if dst has negation and fw" ) );
+        add( new splitIfSrcNegAndFw( "split rule if src has negation and fw" ) );
+        add( new splitIfDstNegAndFw( "split rule if dst has negation and fw" ) );
 
         add( new SrcNegation(                "process negation in Src"  ) );
         add( new DstNegation(                "process negation in Dst"  ) );
@@ -2670,12 +2823,14 @@ void PolicyCompiler_ipt::compile()
 
         add( new specialCaseWithFW1(  "special case with firewall"      ) );
 
-        add( new decideOnChainIfFW( "decide on chain if Src or Dst has fw" ) );
-
+        add( new decideOnChainIfDstFW( "decide on chain if Dst has fw" ) );
         add( new splitIfSrcFWNetwork("split rule if src has a net fw has interface on" ) );
+
+        add( new decideOnChainIfSrcFW( "decide on chain if Src has fw" ) );
         add( new splitIfDstFWNetwork("split rule if dst has a net fw has interface on" ) );
 
 
+
         add( new specialCaseWithFW2( "replace fw with its interfaces in the special case when src==dst==fw" ) );
 
 /* behavior of processors ExpandMultiple... has been changed in
@@ -2726,8 +2881,10 @@ void PolicyCompiler_ipt::compile()
         add( new ConvertToAtomicForIntervals("convert to atomic rules by interval element") );
         add( new convertInterfaceIdToStr("prepare interface assignments"   ) );
         add( new optimize3(                  "optimization 3"              ) );
-        add( new PrintRule(                  "generate iptables code"      ) );
-        add( new simplePrintProgress(                                      ) );
+
+        add( addPrintRuleProcessor() );
+
+        add( new simplePrintProgress());
 
         runRuleProcessors();
 
@@ -2749,7 +2906,7 @@ string PolicyCompiler_ipt::debugPrintRule(Rule *r)
     RuleElementInterval *intrel=rule->getWhen();
 
     string       iface_id = rule->getInterfaceId();
-    Interface *rule_iface = fw_interfaces[iface_id];
+//    Interface *rule_iface = fw_interfaces[iface_id];
 
     ostringstream str;
 
@@ -2840,5 +2997,49 @@ string PolicyCompiler_ipt::debugPrintRule(Rule *r)
 
 void PolicyCompiler_ipt::epilog()
 {
+    if (fwopt->getBool("use_iptables_restore"))
+    {
+        output << "#" << endl;
+    }
     cout << _(" Policy compiled successfully \n") << flush;
 }
+
+
+PolicyCompiler_ipt::PrintRule* PolicyCompiler_ipt::addPrintRuleProcessor()
+{
+    if(printRule==NULL)
+    {
+        if (fwopt->getBool("use_iptables_restore"))
+        {
+            if (have_dynamic_interfaces)
+                printRule=new PrintRuleIptRstEcho("generate code for iptables-restore using echo");
+            else printRule=new PrintRuleIptRst("generate code for iptables-restore");
+        } else   printRule=new PrintRule("generate shell script"    );
+        printRule->setContext(this);
+    }
+    return printRule;
+}
+
+string PolicyCompiler_ipt::flushAndSetDefaultPolicy()
+{
+    string res="";
+
+    addPrintRuleProcessor();
+    res += printRule->_flushAndSetDefaultPolicy();
+    res += printRule->_printOptionalGlobalRules();
+
+    return res;
+}
+
+string PolicyCompiler_ipt::commit()
+{
+    string res="";
+
+    addPrintRuleProcessor();
+
+    res += printRule->_commit();
+
+    return res;
+}
+
+
