@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.h,v 1.4 2004/09/13 02:40:02 vkurland Exp $
+  $Id: PolicyCompiler_ipt.h,v 1.8 2004/12/29 07:56:23 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -56,6 +56,16 @@ namespace fwcompiler {
 
 	protected:
 
+        class PrintRule;
+
+        PolicyCompiler_ipt::PrintRule *printRule;
+        bool   have_dynamic_interfaces;
+
+        
+        PolicyCompiler_ipt::PrintRule* addPrintRuleProcessor();
+
+        std::string getInterfaceVarName(libfwbuilder::FWObject *iface);
+        
 	/**
 	 * Add some predefined rules controlled by checkboxes in
 	 * firewall settings dialog
@@ -346,12 +356,19 @@ namespace fwcompiler {
         };
 
 	/**
-	 * decides what chain this rule should go to if Src or Dst contains
-         * firewall object. This is a simple case and we need to set chain
-         * before we try to split the rule if it contains network the firewall
-         * has interface on (splitIfSrcFWNetwork / splitIfDstFWNetwork).
+	 * decides what chain this rule should go to if Src contains
+	 * firewall object. This is a simple case and we need to set
+	 * chain before we try to split the rule if it contains
+	 * network the firewall has interface on (splitIfSrcFWNetwork
+	 * / splitIfDstFWNetwork).
 	 */
-        DECLARE_POLICY_RULE_PROCESSOR(decideOnChainIfFW);
+        DECLARE_POLICY_RULE_PROCESSOR(decideOnChainIfSrcFW);
+
+	/**
+	 * Similar to the above, except it decides what chain this
+	 * rule should go to if Dst contains firewall object.
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(decideOnChainIfDstFW);
 
 	/**
 	 * This processor takes care of a special case where a rule
@@ -494,8 +511,6 @@ namespace fwcompiler {
         DECLARE_POLICY_RULE_PROCESSOR(optimize2);
         friend class PolicyCompiler_ipt::optimize2;
 
-        class PrintRule;
-
         /**
          * remove duplicate rules
          */
@@ -569,11 +584,18 @@ namespace fwcompiler {
 	 */
         class PrintRule : public PolicyRuleProcessor
         {
+            protected:
+            
             bool                             init;
             bool                             print_once_on_top;
             std::string                      current_rule_label;
             std::map<const std::string,bool> chains;
 
+            virtual std::string _createChain(const std::string &chain);
+            virtual std::string _startRuleLine();
+            virtual std::string _endRuleLine();
+            virtual std::string _printRuleLabel(libfwbuilder::PolicyRule *r);
+
             virtual std::string _printSrcService(libfwbuilder::RuleElementSrv  *o);
             virtual std::string _printDstService(libfwbuilder::RuleElementSrv  *o);
             virtual std::string _printProtocol(libfwbuilder::Service *srv);
@@ -605,6 +627,11 @@ namespace fwcompiler {
 
             PrintRule(const std::string &name);
             virtual std::string _printGlobalLogParameters();
+            virtual std::string _printOptionalGlobalRules();
+            virtual std::string _flushAndSetDefaultPolicy();
+            virtual std::string _commit();
+            virtual std::string _quote(const std::string &s);
+            
             virtual bool processNext();
 
             std::string PolicyRuleToString(libfwbuilder::PolicyRule *r);
@@ -612,6 +639,40 @@ namespace fwcompiler {
         };
         friend class PolicyCompiler_ipt::PrintRule;
 
+        class PrintRuleIptRst : public PrintRule
+        {
+            virtual std::string _createChain(const std::string &chain);
+            virtual std::string _startRuleLine();
+            virtual std::string _endRuleLine();
+            virtual std::string _printRuleLabel(libfwbuilder::PolicyRule *r);
+
+            public:
+            PrintRuleIptRst(const std::string &name) : PrintRule(name) {};
+            virtual std::string _flushAndSetDefaultPolicy();
+            virtual std::string _commit();
+            virtual std::string _quote(const std::string &s);
+
+            virtual bool processNext();
+        };
+        friend class PolicyCompiler_ipt::PrintRuleIptRst;
+
+        class PrintRuleIptRstEcho : public PrintRuleIptRst
+        {
+            virtual std::string _createChain(const std::string &chain);
+            virtual std::string _startRuleLine();
+            virtual std::string _endRuleLine();
+
+            public:
+            PrintRuleIptRstEcho(const std::string &name) : PrintRuleIptRst(name) {};
+            virtual std::string _flushAndSetDefaultPolicy();
+            virtual std::string _commit();
+            virtual std::string _quote(const std::string &s);
+
+            virtual bool processNext();
+        };
+        friend class PolicyCompiler_ipt::PrintRuleIptRstEcho;
+
+
 	virtual std::string myPlatformName();
 
         /**
@@ -627,13 +688,19 @@ namespace fwcompiler {
 
 	PolicyCompiler_ipt(libfwbuilder::FWObjectDatabase *_db,
 			  const std::string &fwname,
-                fwcompiler::OSConfigurator *_oscnf) : PolicyCompiler(_db,fwname,_oscnf) {}
+                fwcompiler::OSConfigurator *_oscnf) : PolicyCompiler(_db,fwname,_oscnf)
+            {have_dynamic_interfaces=false; printRule=NULL;}
 
 
 	virtual int  prolog();
 	virtual void compile();
 	virtual void epilog();
-	
+
+        void setHaveDynamicInterfaces(bool f) { have_dynamic_interfaces=f; }
+        
+        std::string flushAndSetDefaultPolicy();
+        std::string commit();
+
 	static std::string getNewTmpChainName(libfwbuilder::PolicyRule *rule);
 	static std::string getNewChainName(libfwbuilder::PolicyRule *rule,libfwbuilder::Interface *rule_iface);
 
