@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipt.cpp,v 1.31 2004/09/28 05:21:01 vkurland Exp $
+  $Id: ipt.cpp,v 1.40 2005/04/10 05:55:20 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -35,9 +35,11 @@
 #include <iostream>
 #include <algorithm>
 #include <functional>
+#include <stdexcept>
 
 #ifndef _WIN32
 #  include <unistd.h>
+#  include <pwd.h>
 #else
 #  include <direct.h>
 #  include <stdlib.h>
@@ -85,6 +87,7 @@ static int              dl             = 0;
 static int              drp            = -1;
 static int              drn            = -1;
 static int              verbose        = 0;
+static bool             have_dynamic_interfaces = false;
 
 class UpgradePredicate: public XMLTools::UpgradePredicate
 {
@@ -96,7 +99,30 @@ class UpgradePredicate: public XMLTools::UpgradePredicate
     }
 };
 
-    
+string addPrologScript(bool nocomment,const string &script)
+{
+    string res="";
+
+    if ( !nocomment )
+    {
+        res += "\n";
+        res += "#\n";
+        res += "# Prolog script\n";
+        res += "#\n";
+    }
+
+    res += script;
+    res += "\n";
+
+    if ( !nocomment )
+    {
+        res += "#\n";
+        res += "# End of prolog script\n";
+        res += "#\n";
+    }
+    return res;
+}
+
 void usage(const char *name)
 {
     cout << _("Firewall Builder:  policy compiler for Linux 2.4.xx iptables") << endl;
@@ -242,6 +268,7 @@ int main(int argc, char * const *argv)
  * it must be dynamic and should not have a child IPv4 or
  * physAddress object
  */
+
                 if (n!=iface->getName().length()-1)
                 {
                     char errstr[256];
@@ -277,6 +304,8 @@ _("Wildcard interface '%s' should not have a physcal address object attached to
 
             if ( iface->isDyn())  
             {
+                have_dynamic_interfaces=true;
+
                 iface->setBool("use_var_address",true);
 
                 list<FWObject*> l3=iface->getByType(IPv4::TYPENAME);
@@ -335,10 +364,12 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 	string pfctl_dbg=(debug)?"-v":"";
 
 	OSConfigurator_linux24 *oscnf=NULL;
-	if (fw->getStr("host_OS")=="linux24" || fw->getStr("host_OS")=="linksys")
+        string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
+        if ( family=="linux24" )
             oscnf=new OSConfigurator_linux24(FWObjectDatabase::db , fwobjectname);
+
 	if (oscnf==NULL)
-	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS"));
+	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
 
         oscnf->prolog();
 
@@ -347,6 +378,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 	c.setDebugLevel( dl );
 	c.setDebugRule(  drp );
 	c.setVerbose( (bool)(verbose) );
+        c.setHaveDynamicInterfaces(have_dynamic_interfaces);
 
 	if ( c.prolog() > 0 )
         {
@@ -359,6 +391,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 	n.setDebugLevel( dl );
 	n.setDebugRule(  drn );
 	n.setVerbose( (bool)(verbose) );
+        n.setHaveDynamicInterfaces(have_dynamic_interfaces);
 
 	if ( n.prolog() > 0 ) {
             oscnf->generateCodeForProtocolHandlers(true);
@@ -384,14 +417,20 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 #ifdef _WIN32
         char* user_name=getenv("USERNAME");
 #else
-        char* user_name=getenv("USER");
+        struct passwd *pwd=getpwuid(getuid());
+        assert(pwd);
+        char *user_name=pwd->pw_name;
 #endif
 
-        if (user_name==NULL) {
-            cerr << _("Can't figure out your user name, aborting") << endl;
-            exit(1);
+        if (user_name==NULL)
+        {
+            user_name=getenv("LOGNAME");
+            if (user_name==NULL)
+            {
+                cerr << _("Can't figure out your user name, aborting") << endl;
+                exit(1);
+            }
         }
-
 /*
  * assemble the script and then perhaps post-process it if it should
  * run on Linksys device with sveasoft firmware
@@ -414,7 +453,8 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         script << "#" << endl;
 
 /* do not put comment in the script if it is intended for linksys */
-        if (fw->getStr("host_OS")!="linksys")
+        bool nocomm=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceBool("/FWBuilderResources/Target/options/suppress_comments");
+        if ( !nocomm )
         {
             string fwcomment=fw->getComment();
             string::size_type n1,n2;
@@ -431,41 +471,123 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         script << shell_dbg << endl;
         script << endl;
 
-        script << "PATH=\"/usr/sbin:/sbin:${PATH}\"" << endl;
+        script << "PATH=\"/sbin:/usr/sbin:/bin:/usr/bin:${PATH}\"" << endl;
         script << "export PATH" << endl;
         script << endl;
 
+        string prolog_place= fw->getOptionsObject()->getStr("prolog_place");
+        if (prolog_place == "") prolog_place="top";
+
+        if (prolog_place == "top")
+        {
+            script <<
+                addPrologScript(nocomm, 
+                                fw->getOptionsObject()->getStr("prolog_script"));
+        }
+
 	script << oscnf->getCompiledScript();
 
         script << endl;
 
-//        script << "cd " << firewall_dir << " || exit 1" << endl << endl;
-        script << "log \"";
+        if (prolog_place == "after_interfaces")
+        {
+            script <<
+                addPrologScript(nocomm, 
+                                fw->getOptionsObject()->getStr("prolog_script"));
+        }
+
+        script << "log '";
         script << _("Activating firewall script generated ")
                << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
                << user_name;
-        script << "\"" << endl;
+        script << "'" << endl;
 
 	script << endl;
 
-        if ( options->getBool("accept_established") ) 
+        if (options->getBool("use_iptables_restore"))
         {
-            script << "$IPTABLES -A INPUT   -m state --state ESTABLISHED,RELATED -j ACCEPT"
-                   << endl;
-            script << "$IPTABLES -A OUTPUT  -m state --state ESTABLISHED,RELATED -j ACCEPT"
-                   << endl;
-            script << "$IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT"
-                   << endl;
-            script << endl;
+            if (have_dynamic_interfaces)
+            {
+                script << "(" << endl;
+                script << c.flushAndSetDefaultPolicy();
+
+                if (prolog_place == "after_flush")
+                {
+                    script << addPrologScript(nocomm, 
+                                 fw->getOptionsObject()->getStr("prolog_script"));
+                }
+
+                script << c.getCompiledScript();
+                script << c.commit();
+                script << n.flushAndSetDefaultPolicy();
+                script << n.getCompiledScript();
+                script << n.commit();
+                script << "#" << endl;
+                script << ") | $IPTABLES_RESTORE" << endl;
+            } else
+            {
+                script << "cat << EOF | $IPTABLES_RESTORE" << endl;
+                script << c.flushAndSetDefaultPolicy();
+
+                if (prolog_place == "after_flush")
+                {
+                    script << addPrologScript(nocomm, 
+                                 fw->getOptionsObject()->getStr("prolog_script"));
+                }
+
+                script << c.getCompiledScript();
+                script << c.commit();
+                script << n.flushAndSetDefaultPolicy();
+                script << n.getCompiledScript();
+                script << n.commit();
+                script << "#" << endl;
+                script << "EOF" << endl;
+            }
+        } else
+        {
+/* the order here is historical: I always installed default policy and
+ * added optional global rules before adding NAT and added normal
+ * policy after NAT
+ */
+            script << c.flushAndSetDefaultPolicy();
+
+            if (prolog_place == "after_flush")
+            {
+                script << addPrologScript(nocomm, 
+                             fw->getOptionsObject()->getStr("prolog_script"));
+            }
+
+            script << n.flushAndSetDefaultPolicy();
+            script << n.getCompiledScript();
+            script << n.commit();
+            script << c.getCompiledScript();
+            script << c.commit();
         }
 
 
-	script << n.getCompiledScript();
-	script << c.getCompiledScript();
+
 
         oscnf->epilog();
 	script << oscnf->getCompiledScript();
 
+        if ( !nocomm )
+        {
+            script << endl;
+            script << "#" << endl;
+            script << "# Epilog script" << endl;
+            script << "#" << endl;
+        }
+
+        string post_hook= fw->getOptionsObject()->getStr("epilog_script");
+        script << post_hook << endl;
+
+        if ( !nocomm )
+        {
+            script << endl;
+            script << "# End of epilog script" << endl;
+            script << "#" << endl;
+        }
+
 	script << endl;
 
         string sbuf = script.str();
@@ -512,10 +634,13 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         }
 #endif
 
+	ofstream fw_file;
+        fw_file.exceptions(ofstream::eofbit|ofstream::failbit|ofstream::badbit);
+
 #ifdef _WIN32
-	ofstream fw_file(fw_file_name.c_str(), ios::out|ios::binary);
+        fw_file.open(fw_file_name.c_str(), ios::out|ios::binary);
 #else
-	ofstream fw_file(fw_file_name.c_str());
+        fw_file.open(fw_file_name.c_str());
 #endif
 
         fw_file << sbuf << endl;
@@ -529,27 +654,25 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         
         return 0;
 
-    } catch(FWException &ex)  {
+    } catch(const FWException &ex)  {
 	cerr << ex.toString() << endl;
         return 1;
-    } catch (std::string s) {
-	cerr << s;
+#if __GNUC__ >= 3
+/* need to check version because std::ios::failure does not seem to be
+ * supported in gcc 2.9.5 on FreeBSD 4.10 */
+    } catch (const std::ios::failure &e) {
+        cerr << "Error while opening or writing to the output file" << endl;
         return 1;
-    } catch (std::exception ex) {
-	cerr << ex.what();
+#endif
+    } catch (const std::string &s) {
+	cerr << s << endl;
+        return 1;
+    } catch (const std::exception &ex) {
+	cerr << ex.what() << endl;
         return 1;
     } catch (...) {
-	cerr << _("Unsupported exception");
+	cerr << _("Unsupported exception") << endl;
         return 1;
     }
 
 }
-
-
-
-
-
-
-
-
-
