@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: pf.cpp,v 1.16 2004/09/28 05:21:01 vkurland Exp $
+  $Id: pf.cpp,v 1.23 2005/04/14 06:30:37 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,9 +36,11 @@
 #include <sstream>
 #include <algorithm>
 #include <functional>
+#include <stdexcept>
 
 #ifndef _WIN32
 #  include <unistd.h>
+#  include <pwd.h>
 #else
 #  include <direct.h>
 #  include <stdlib.h>
@@ -261,7 +263,6 @@ int main(int argc, char * const *argv)
             Interface *iface=dynamic_cast<Interface*>(*i);
             assert(iface);
 
-            string::size_type n;
             if ( iface->isDyn())  
             {
                 list<FWObject*> l3=iface->getByType(IPv4::TYPENAME);
@@ -334,11 +335,19 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
  * Process firewall options, build OS network configuration script
  */
 	OSConfigurator *oscnf=NULL;
-	if (fw->getStr("host_OS")=="solaris")   oscnf=new OSConfigurator_solaris(FWObjectDatabase::db , fwobjectname);
-	if (fw->getStr("host_OS")=="openbsd")   oscnf=new OSConfigurator_openbsd(FWObjectDatabase::db , fwobjectname);
-	if (fw->getStr("host_OS")=="freebsd")   oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+        string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
+
+	if (family=="solaris")
+            oscnf=new OSConfigurator_solaris(FWObjectDatabase::db , fwobjectname);
+
+	if (family=="openbsd")
+            oscnf=new OSConfigurator_openbsd(FWObjectDatabase::db , fwobjectname);
+
+	if (family=="freebsd")
+            oscnf=new OSConfigurator_freebsd(FWObjectDatabase::db , fwobjectname);
+
 	if (oscnf==NULL)
-	    throw FWException("Unrecognized host OS "+fw->getStr("host_OS"));
+	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
 
 	oscnf->prolog();
 
@@ -353,7 +362,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 	n.setDebugRule(  drn );
 	n.setVerbose( verbose );
 
-	bool     have_nat;
+	bool     have_nat=false;
 	if ( n.prolog() > 0 ) 
         {
 	    have_nat=true;
@@ -382,10 +391,13 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
  * now write generated scripts to files
  */
 
+        ofstream pf_file;
+        pf_file.exceptions(ofstream::eofbit|ofstream::failbit|ofstream::badbit);
+
 #ifdef _WIN32
-	ofstream pf_file(pf_file_name.c_str(), ios::out|ios::binary);
+        pf_file.open(pf_file_name.c_str(), ios::out|ios::binary);
 #else
-	ofstream pf_file(pf_file_name.c_str());
+        pf_file.open(pf_file_name.c_str());
 #endif
 
         pf_file << endl;
@@ -473,20 +485,6 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
         if ( ! options->getStr("pf_optimization").empty() )
             pf_file << "set optimization "
                     << options->getStr("pf_optimization") << endl;
@@ -549,17 +547,27 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 #ifdef _WIN32
         char* user_name=getenv("USERNAME");
 #else
-        char* user_name=getenv("USER");
+        struct passwd *pwd=getpwuid(getuid());
+        assert(pwd);
+        char *user_name=pwd->pw_name;
 #endif
-        if (user_name==NULL) {
-            cerr << _("Can't figure out your user name, aborting") << endl;
-            exit(1);
+        if (user_name==NULL)
+        {
+            user_name=getenv("LOGNAME");
+            if (user_name==NULL)
+            {
+                cerr << _("Can't figure out your user name, aborting") << endl;
+                exit(1);
+            }
         }
 
+	ofstream fw_file;
+        fw_file.exceptions(ofstream::eofbit|ofstream::failbit|ofstream::badbit);
+
 #ifdef _WIN32
-	ofstream fw_file(fw_file_name.c_str(), ios::out|ios::binary);
+        fw_file.open(fw_file_name.c_str(), ios::out|ios::binary);
 #else
-	ofstream fw_file(fw_file_name.c_str());
+        fw_file.open(fw_file_name.c_str());
 #endif
 	fw_file << "#!/bin/sh " << shell_dbg << endl;
 
@@ -593,11 +601,11 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
         fw_file << endl;
 
-        fw_file << "log \"";
+        fw_file << "log '";
         fw_file << _("Activating firewall script generated ")
                << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
                << user_name;
-        fw_file << "\"" << endl;
+        fw_file << "'" << endl;
 
 	fw_file << endl;
 
@@ -606,13 +614,52 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
         fw_file << endl 
                 << "$PFCTL -d " << endl
-                << "$PFCTL -F all " << endl;
+                << "$PFCTL -F nat" << endl
+                << "$PFCTL -F rules" << endl
+                << "$PFCTL -F Sources" << endl
+                << "$PFCTL -F Tables" << endl;
+
+/*
+ *  we add prolog and epilog to the activation shell script rather
+ *  than to the .conf file. This is more flexible since user can
+ *  execute some shell commands, as well as add any policy and/or nat
+ *  rules by putting them into their .conf file and loading them from
+ *  prolog or epilog script. Because of this, prolog is added after
+ *  all rules are flushed.
+ */ 
+        fw_file << endl;
+        fw_file << "#" << endl;
+        fw_file << "# Prolog script" << endl;
+        fw_file << "#" << endl;
+        
+        string pre_hook= fw->getOptionsObject()->getStr("prolog_script");
+        fw_file << pre_hook << endl;
+
+        fw_file << "#" << endl;
+        fw_file << "# End of prolog script" << endl;
+        fw_file << "#" << endl;
 
 	fw_file << endl;
+
         fw_file << "$PFCTL " << pfctl_dbg << pfctl_f_option
                 << "${FWDIR}/" << pf_file_name << endl;
 	fw_file << "$PFCTL -e" << endl;
         
+        fw_file << endl;
+        fw_file << "#" << endl;
+        fw_file << "# Epilog script" << endl;
+        fw_file << "#" << endl;
+
+        string post_hook= fw->getOptionsObject()->getStr("epilog_script");
+        fw_file << post_hook << endl;
+
+        fw_file << endl;
+        fw_file << "# End of epilog script" << endl;
+        fw_file << "#" << endl;
+
+	fw_file << endl;
+        fw_file.close();
+
 #ifdef _WIN32
         _chmod(fw_file_name.c_str(),_S_IREAD|_S_IWRITE);
 #else
@@ -621,13 +668,20 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         
         return 0;
 
-    } catch(FWException &ex)  {
+    } catch(const FWException &ex)  {
 	cerr << ex.toString() << endl;
         return 1;
-    } catch (std::string s) {
+#if __GNUC__ >= 3
+/* need to check version because std::ios::failure does not seem to be
+ * supported in gcc 2.9.5 on FreeBSD 4.10 */
+    } catch (const std::ios::failure &e) {
+        cerr << "Error while opening or writing to the output file" << endl;
+        return 1;
+#endif
+    } catch (const std::string &s) {
 	cerr << s;
         return 1;
-    } catch (std::exception ex) {
+    } catch (const std::exception &ex) {
 	cerr << ex.what();
         return 1;
     } catch (...) {
