@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: OSConfigurator_freebsd.cpp,v 1.1 2004/05/11 06:06:29 vkurland Exp $
+  $Id: OSConfigurator_freebsd.cpp,v 1.3 2005/04/07 15:44:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -42,6 +42,12 @@ using namespace std;
 
 string OSConfigurator_freebsd::myPlatformName() { return "FreeBSD"; }
 
+string OSConfigurator_freebsd::getInterfaceVarName(FWObject *iface)
+{
+    return string("i_") + iface->getName();
+}
+
+
 void OSConfigurator_freebsd::processFirewallOptions() 
 {
     FWOptions* options=fw->getOptionsObject();
@@ -188,6 +194,23 @@ void  OSConfigurator_freebsd::printPathForAllTools(const string &os)
     output << endl;
     output << endl;
 
+    if (options->getBool("dynAddr"))
+    {
+        output << "getaddr() {"                       << endl;
+        output << "  intf=$1"                         << endl;
+        output << "  varname=$2"                      << endl;
+        output << "  L=`ifconfig $1 | grep 'inet '`"  << endl;
+        output << "  if [ -z \"$L\" ]; then"          << endl;
+        output << "      L=\"inet 0.0.0.0/32\""       << endl;
+        output << "  fi"                              << endl;
+        output << "  set $L"                          << endl;
+        output << "  a=$2"                            << endl;
+        output << "  eval \"$varname=$a\""            << endl;
+        output << "}"                                 << endl;
+        output << endl;
+        output << endl;
+    }
+
     output << "IPF=\""    + path_ipf    + "\"\n";
     output << "IPNAT=\""  + path_ipnat  + "\"\n";
     output << "IPFW=\""   + path_ipfw   + "\"\n";
@@ -196,6 +219,33 @@ void  OSConfigurator_freebsd::printPathForAllTools(const string &os)
     output << "LOGGER=\"" + path_logger + "\"\n";
     output << endl;
 
+    if (options->getBool("dynAddr"))
+    {
+/*
+ * get addresses of dynamic interfaces
+ */
+        FWObjectTypedChildIterator j=fw->findByType(Interface::TYPENAME);
+        for ( ; j!=j.end(); ++j ) 
+        {
+            Interface *iface=Interface::cast(*j);
+        
+            if ( iface->isDyn() )
+            {
+/* if interface name ends with '*', this is a wildcard interface. Do
+ * not get its address at this time. 
+ *
+ * Do we support wildcard interfaces on *BSD at all ?
+ */
+                if (iface->getName().find("*")==string::npos)
+                    output << "getaddr "
+                           << iface->getName()
+                           << "  "
+                           << getInterfaceVarName(iface)
+                           << endl;
+            }
+        }
+    }
+
     output << endl;
 }
 
