@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw_writers.cpp,v 1.4 2004/05/15 22:28:44 vkurland Exp $
+  $Id: PolicyCompiler_ipfw_writers.cpp,v 1.5 2004/12/31 07:31:45 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -24,6 +24,7 @@
 */
 
 #include "PolicyCompiler_ipfw.h"
+#include "OSData.h"
 
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/RuleElement.h"
@@ -37,6 +38,7 @@
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/FWOptions.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/Resources.h"
 
 #include <iostream>
 
@@ -69,12 +71,38 @@ using namespace std;
 void PolicyCompiler_ipfw::PrintRule::_printProtocol(Service *srv)
 {
 
-    if ( ! srv->isAny() && ! CustomService::isA(srv) && srv->getProtocolName()!="ip") 
+    if ( srv->isAny() || srv->getProtocolName()=="ip") 
     {
-	compiler->output << srv->getProtocolName();
-	compiler->output << " ";
-    } else
 	compiler->output << "all ";
+        return;
+    }
+
+    if ( CustomService::isA(srv) )
+    {
+	string cscode=
+            CustomService::cast(srv)->getCodeForPlatform( compiler->myPlatformName() );
+/*
+ * This is really a hack
+ *
+ * CustomService object does not imply any specific protocol so
+ * generally we can't add protocol name if Custom Service object is
+ * used. However, there is one particular case where we have to
+ * specify protocol 'tcp' instead of 'all', that is when CustomService
+ * object is used to add an option 'established' which is only valid
+ * for tcp protocol.
+ *
+ * Perhaps better solution would be to add optional protocol
+ * specification to the CustomService object
+ */
+        if (cscode=="established")
+            compiler->output << "tcp ";
+        else
+            compiler->output << "all ";
+        return;
+    }
+
+    compiler->output << srv->getProtocolName();
+    compiler->output << " ";
 }
 
 
@@ -376,14 +404,20 @@ PolicyCompiler_ipfw::PrintRule::PrintRule(const std::string &name) : PolicyCompi
 bool PolicyCompiler_ipfw::PrintRule::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
-    FWOptions  *ruleopt =rule->getOptionsObject();
+    FWOptions*  options = compiler->fw->getOptionsObject();
+    FWOptions  *ruleopt = rule->getOptionsObject();
 
     tmp_queue.push_back(rule);
 
+/* need to quote $IPFW because it may contain space, this happens on
+ * Mac more often than anywhere else */
+
+    string quote = "\"";
 
     if (print_once_on_top) 
     {
-        compiler->output << "$IPFW add 1 check-state ip from any to any"  << endl;
+        compiler->output << quote << "$IPFW" << quote
+                         << " add 1 check-state ip from any to any"  << endl;
         compiler->output << endl;
 
         print_once_on_top=false;
@@ -418,7 +452,8 @@ bool PolicyCompiler_ipfw::PrintRule::processNext()
     RuleElementSrv *srvrel=rule->getSrv();
     Service        *srv   =compiler->getFirstSrv(rule);  assert(srv);
 
-    compiler->output << "$IPFW add " << rule->getInt("ipfw_num") << " ";
+    compiler->output <<  quote << "$IPFW" << quote
+                     << " add " << rule->getInt("ipfw_num") << " ";
 
     _printAction(rule);
 
