@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf.cpp,v 1.6 2004/09/19 06:20:12 vkurland Exp $
+  $Id: PolicyCompiler_pf.cpp,v 1.7 2005/04/04 07:00:54 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -637,6 +637,54 @@ bool PolicyCompiler_pf::splitIfInterfaceInRE::processNext()
     return true;
 }
 
+bool PolicyCompiler_pf::separateSrcPort::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+
+    RuleElementSrv *rel= rule->getSrv();
+
+    if (rel->size()==1) {
+	tmp_queue.push_back(rule);
+	return true;
+    }
+
+    list<Service*> services;
+    for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
+	    
+	FWObject *o= *i;
+	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	Service *s=Service::cast(o);
+	assert(s!=NULL);
+
+	if ( TCPService::isA(s) || UDPService::isA(s) ) {
+            int srs=s->getInt("src_range_start");
+            int sre=s->getInt("src_range_end");
+
+            compiler->normalizePortRange(srs,sre);
+
+            if (srs!=0 || sre!=0) {
+                PolicyRule *r= PolicyRule::cast(
+                    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+                compiler->temp_ruleset->add(r);
+                r->duplicate(rule);
+                RuleElementSrv *nsrv=r->getSrv();
+                nsrv->clearChildren();
+                nsrv->addRef( s );
+                tmp_queue.push_back(r);
+                services.push_back(s);
+            } 
+        }
+    }
+    for (list<Service*>::iterator i=services.begin(); i!=services.end(); i++) 
+	rel->removeRef( (*i) );
+
+    if (!rel->isAny())
+	tmp_queue.push_back(rule);
+
+    return true;
+}
+
+
 FWObject* PolicyCompiler_pf::createTables::findTable(RuleElement *re)
 {
     PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
@@ -821,6 +869,7 @@ void PolicyCompiler_pf::compile()
 	add( new addressRanges(         "expand address range objects"        ) );
 	add( new splitServices(         "split rules with different protocols") );
 	add( new separateTCPWithFlags(  "separate TCP services with flags"    ) );
+        add( new separateSrcPort("split on TCP and UDP with source ports"));
 	add( new verifyCustomServices(  "verify custom services for this platform") );
 //	add( new ProcessScrubOption(    "process 'scrub' option"         ) );
 	add( new SpecialServices(       "check for special services"     ) );
